set.seed(54749014)
x1 <- runif(n, 1, 6)
x2 <- runif(n, 0, 2*pi)

for(r in 1:reps){
  set.seed(r + seed.offset)
  
  beta.xi <- 0
  beta.omega <- 0
  beta.alpha <- 0

  
  eta.xi <- beta.xi + 1 / x1 - mean(1 / x1) + cos(2*x2) * sqrt(x2) - mean(cos(2*x2) * sqrt(x2))
  eta.omega <- beta.omega + 0.7 * x1 - 0.1 * x1^2 - mean(0.7 * x1 - 0.1 * x1^2) + cos(x2) - mean(cos(x2))
  eta.alpha <- beta.alpha + 1 * (log(x1) - mean(log(x1)) + sin(x2) - mean(sin(x2)))
  
  
  y <- rep(0, n)
  for(i in 1:n){
    dp <- cp2dp(cp = c(eta.xi[i], exp(eta.omega[i]), get.gamma(eta.alpha[i])), family = "SN")
    y[i] <- rsn(1, dp = dp)
  }
  
  data.name <- paste(title)
  
  dat <- data.frame(y = y, x1 = x1, x2 = x2, x0 = rep(1, n))
  write.table(dat, col.names = TRUE, row.names = FALSE, sep = "\t", quote = FALSE,
              file = paste(directory, data.name, ".raw", sep = ""))
  
  sim.name <- paste(title)
  
  filename <- create.file.for.bayesx(directory = directory,
                                     sim.name = sim.name,
                                     data.name = data.name,
                                     distr.name = "normal2",
                                     equation.names = c("sigma", "mu"),
                                     response.name = "y",
                                     form.names = c("const+x1(pspline)+x2(pspline)", "const+x1(pspline)+x2(pspline)"),
                                     seed = r + seed.offset,
                                     iterations = 12000,
                                     burnin = 2000,
                                     step = 10,
                                     predict = TRUE,
                                     getsample = FALSE)
  
  system(paste("~/bsx_fin/./bayesx", filename))
  #system(paste("C:/fbsx_ohne_bivst/bin/Debug/bayesx.exe", filename))
  
  dat.xi.lin <- read.table("sim_mu_MAIN_mu_REGRESSION_y_LinearEffects.res", header = TRUE)
  dat.xi.x1 <- read.table("sim_mu_MAIN_mu_REGRESSION_y_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.xi.x1.p <- read.table("sim_mu_MAIN_mu_REGRESSION_y_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.xi.x1.v <- read.table("sim_mu_MAIN_mu_REGRESSION_y_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.xi.x2 <- read.table("sim_mu_MAIN_mu_REGRESSION_y_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.xi.x2.p <- read.table("sim_mu_MAIN_mu_REGRESSION_y_nonlinear_pspline_effect_of_x2_param.res", header = TRUE)
  dat.xi.x2.v <- read.table("sim_mu_MAIN_mu_REGRESSION_y_variance_of_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.om.lin <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_LinearEffects.res", header = TRUE)
  dat.om.x1 <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.om.x1.p <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.om.x1.v <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.om.x2 <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.om.x2.p <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_nonlinear_pspline_effect_of_x2_param.res", header = TRUE)
  dat.om.x2.v <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_variance_of_nonlinear_pspline_effect_of_x2.res", header = TRUE)

  pred <- read.table("sim_mu_MAIN_mu_REGRESSION_y_predict.res", header = TRUE)
  pred.dic <- read.table("sim_mu_MAIN_mu_REGRESSION_y_predict_DIC.res", header = TRUE); pred.waic <- read.table("sim_mu_MAIN_mu_REGRESSION_y_predict_WAIC.res", header = TRUE)
  
  #crit <- get.crit()
  
  save(y,
       x1,
       x2,
       #crit,
       dat.xi.lin,
       dat.xi.x1,
       dat.xi.x1.p,
       dat.xi.x1.v,
       dat.xi.x2,
       dat.xi.x2.p,
       dat.xi.x2.v,
       dat.om.lin,
       dat.om.x1,
       dat.om.x1.p,
       dat.om.x1.v,
       dat.om.x2,
       dat.om.x2.p,
       dat.om.x2.v,

       pred,
       pred.dic, pred.waic,
       file = paste(title, r + seed.offset, ".RData", sep = ""))
}