set.seed(54749014)
x1 <- runif(n, 1, 6)
x2 <- runif(n, 0, 2*pi)

for(r in 1:reps){
  set.seed(r + seed.offset)
  
  beta.xi1 <- 0
  beta.xi2 <- 0
  beta.omega1 <- 0
  beta.omega2 <- - 0.5
  beta.rho <- 0
  beta.alpha1 <- 0
  beta.alpha2 <- 0
  beta.nu <- 2
  
  eta.xi1 <- beta.xi1 + 1 / x1 - mean(1 / x1) + cos(2*x2) * sqrt(x2) - mean(cos(2*x2) * sqrt(x2))
  eta.xi2 <- beta.xi2 + 2*x1 - 3*log(x1)^2 - mean(2*x1 - 3*log(x1)^2) + 0.1 * sin(x2) * exp(0.5 * x2) - mean(0.1 * sin(x2) * exp(0.5 * x2))
  eta.omega1 <- beta.omega1 + 0.7 * x1 - 0.1 * x1^2 - mean(0.7 * x1 - 0.1 * x1^2) + 0.5 * sin(2*x2) - mean(0.5 * sin(2*x2))
  eta.omega2 <- beta.omega2 + 0.025 * x1^3 - 0.15 * x1^2 - mean(0.025 * x1^3 - 0.15 * x1^2) + 0.1 * cos(x2) * x2 - mean(0.1 * cos(x2) * x2)
  eta.rho <- beta.rho + 0.6 * log(x1) - mean(0.6 * log(x1)) + 0.5 * cos(x2) - mean(0.5*cos(x2))
  eta.alpha1 <- beta.alpha1 + 1 * (log(x1 - 0.1 * x1^2) - mean(log(x1 - 0.1 * x1^2)) + 0.5 * sin(x2) - mean(0.5 * sin(x2)))
  eta.alpha2 <- beta.alpha2 + 1 * (1.8 / sqrt(x1) - mean(1.8 / sqrt(x1)) + tanh(x2^2 / 2 / pi) - mean(tanh(x2^2 / 2 / pi)))
  eta.nu <- beta.nu + 0.2 * x1 - mean(0.2 * x1) + 2 / (1 + x2) - mean(2 / (1 + x2))
  
  y1 <- rep(0, n)
  y2 <- rep(0, n)
  for(i in 1:n){
    vls <- rmst(1, xi = c(eta.xi1[i], eta.xi2[i]),
                Omega = matrix(c(exp(eta.omega1[i])^2,
                                 get.rho(eta.rho[i]) * exp(eta.omega1[i]) * exp(eta.omega2[i]),
                                 get.rho(eta.rho[i]) * exp(eta.omega1[i]) * exp(eta.omega2[i]),
                                 exp(eta.omega2[i])^2), nrow = 2),
                alpha = c(eta.alpha1[i], eta.alpha2[i]),
                nu = exp(eta.nu[i]))
    y1[i] <- vls[1]
    y2[i] <- vls[2]
  }
  
  data.name <- paste(title)
  
  dat <- data.frame(y1 = y1, y2 = y2, x1 = x1, x2 = x2, x0 = rep(1, n), x3 = rep(3, n))
  write.table(dat, col.names = TRUE, row.names = FALSE, sep = "\t", quote = FALSE,
              file = paste(directory, data.name, ".raw", sep = ""))
  
  sim.name <- paste(title)
  
  filename <- create.file.for.bayesx(directory = directory,
                                     sim.name = sim.name,
                                     data.name = data.name,
                                     distr.name = "bivt",
                                     equation.names = c("df", "rho", "sigma", "sigma", "mu", "mu"),
                                     response.name = c("y1", "y1", "y2", "y1", "y2", "y1"),
                                     form.names = c("x0(ridge,shrinkagefix,shrinkage=1)+x3(offset)+x1(pspline,difforder=1)+x2(pspline,difforder=1)", "const+x1(pspline)+x2(pspline)", "const+x1(pspline)+x2(pspline)", "const+x1(pspline)+x2(pspline)", "const+x1(pspline)+x2(pspline)", "const+x1(pspline)+x2(pspline)"),
                                     seed = r + seed.offset,
                                     iterations = 12000,
                                     burnin = 2000,
                                     step = 10,
                                     predict = TRUE,
                                     getsample = FALSE)
  
  system(paste("~/bsx_fin/./bayesx", filename))
  #system(paste("C:/fbsx_ohne_bivst/bin/Debug/bayesx.exe", filename))
  
  dat.mu1.lin <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_LinearEffects.res", header = TRUE)
  dat.mu1.x1 <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.mu1.x1.p <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.mu1.x1.v <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.mu1.x2 <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.mu1.x2.p <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_nonlinear_pspline_effect_of_x2_param.res", header = TRUE)
  dat.mu1.x2.v <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_variance_of_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.mu2.lin <- read.table("sim_mu_MAIN_mu_REGRESSION_y2_LinearEffects.res", header = TRUE)
  dat.mu2.x1 <- read.table("sim_mu_MAIN_mu_REGRESSION_y2_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.mu2.x1.p <- read.table("sim_mu_MAIN_mu_REGRESSION_y2_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.mu2.x1.v <- read.table("sim_mu_MAIN_mu_REGRESSION_y2_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.mu2.x2 <- read.table("sim_mu_MAIN_mu_REGRESSION_y2_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.mu2.x2.p <- read.table("sim_mu_MAIN_mu_REGRESSION_y2_nonlinear_pspline_effect_of_x2_param.res", header = TRUE)
  dat.mu2.x2.v <- read.table("sim_mu_MAIN_mu_REGRESSION_y2_variance_of_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.si1.lin <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y1_LinearEffects.res", header = TRUE)
  dat.si1.x1 <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y1_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.si1.x1.p <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y1_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.si1.x1.v <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y1_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.si1.x2 <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y1_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.si1.x2.p <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y1_nonlinear_pspline_effect_of_x2_param.res", header = TRUE)
  dat.si1.x2.v <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y1_variance_of_nonlinear_pspline_effect_of_x2.res", header = TRUE)  
  dat.si2.lin <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y2_LinearEffects.res", header = TRUE)
  dat.si2.x1 <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y2_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.si2.x1.p <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y2_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.si2.x1.v <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y2_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.si2.x2 <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y2_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.si2.x2.p <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y2_nonlinear_pspline_effect_of_x2_param.res", header = TRUE)
  dat.si2.x2.v <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y2_variance_of_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.rho.lin <- read.table("sim_rho_MAIN_rho_REGRESSION_y1_LinearEffects.res", header = TRUE)
  dat.rho.x1 <- read.table("sim_rho_MAIN_rho_REGRESSION_y1_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.rho.x1.p <- read.table("sim_rho_MAIN_rho_REGRESSION_y1_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.rho.x1.v <- read.table("sim_rho_MAIN_rho_REGRESSION_y1_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.rho.x2 <- read.table("sim_rho_MAIN_rho_REGRESSION_y1_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.rho.x2.p <- read.table("sim_rho_MAIN_rho_REGRESSION_y1_nonlinear_pspline_effect_of_x2_param.res", header = TRUE)
  dat.rho.x2.v <- read.table("sim_rho_MAIN_rho_REGRESSION_y1_variance_of_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.df.lin <- read.table("sim_df_MAIN_df_REGRESSION_y1_LinearEffects_ridgepenalty.res", header = TRUE)
  dat.df.x1 <- read.table("sim_df_MAIN_df_REGRESSION_y1_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.df.x1.p <- read.table("sim_df_MAIN_df_REGRESSION_y1_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.df.x1.v <- read.table("sim_df_MAIN_df_REGRESSION_y1_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.df.x2 <- read.table("sim_df_MAIN_df_REGRESSION_y1_nonlinear_pspline_effect_of_x2.res", header = TRUE)
  dat.df.x2.p <- read.table("sim_df_MAIN_df_REGRESSION_y1_nonlinear_pspline_effect_of_x2_param.res", header = TRUE)
  dat.df.x2.v <- read.table("sim_df_MAIN_df_REGRESSION_y1_variance_of_nonlinear_pspline_effect_of_x2.res", header = TRUE)

  pred <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_predict.res", header = TRUE)
  pred.dic <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_predict_DIC.res", header = TRUE)
  pred.waic <- read.table("sim_mu_MAIN_mu_REGRESSION_y1_predict_WAIC.res", header = TRUE)
   
  save(y1,
       y2,
       x1,
       x2,
       dat.mu1.lin,
       dat.mu1.x1,
       dat.mu1.x1.p,
       dat.mu1.x1.v,
       dat.mu1.x2,
       dat.mu1.x2.p,
       dat.mu1.x2.v,
       dat.mu2.lin,
       dat.mu2.x1,
       dat.mu2.x1.p,
       dat.mu2.x1.v,
       dat.mu2.x2,
       dat.mu2.x2.p,
       dat.mu2.x2.v,
       dat.si1.lin,
       dat.si1.x1,
       dat.si1.x1.p,
       dat.si1.x1.v,
       dat.si1.x2,
       dat.si1.x2.p,
       dat.si1.x2.v,
       dat.si2.lin,
       dat.si2.x1,
       dat.si2.x1.p,
       dat.si2.x1.v,
       dat.si2.x2,
       dat.si2.x2.p,
       dat.si2.x2.v,
       dat.rho.lin,
       dat.rho.x1,
       dat.rho.x1.p,
       dat.rho.x1.v,
       dat.rho.x2,
       dat.rho.x2.p,
       dat.rho.x2.v,
       dat.df.lin,
       dat.df.x1,
       dat.df.x1.p,
       dat.df.x1.v,
       dat.df.x2,
       dat.df.x2.p,
       dat.df.x2.v,
       pred,
       pred.dic,
       pred.waic,
       file = paste(title, r + seed.offset, ".RData", sep = ""))
}