set.seed(21395)
x1 <- runif(n, 0, 2*pi)

for(r in 1:reps){
  set.seed(r + seed.offset)
  x2 <- random.eff.id(n, 100)
  eff.x2 <- random.eff.val(x2, 100, 1 / 0.6331101, 4)
  eff.x2 <- eff.x2 - mean(eff.x2[1:100])
  
  beta.mu <- 0
  beta.sigma <- 0
  
  eta.mu <- beta.mu + cos(2*x1) * sqrt(x1) - mean(cos(2*x1) * sqrt(x1)) + eff.x2
  eta.sigma <- beta.sigma + sin(x1) - mean(sin(x1))

  
  y <- rep(0, n)
  for(i in 1:n){
    y[i] <- rnorm(1, eta.mu[i], exp(eta.sigma[i]))
  }
  
  data.name <- paste(title)
  
  dat <- data.frame(y = y, x1 = x1, x2 = x2)
  write.table(dat, col.names = TRUE, row.names = FALSE, sep = "\t", quote = FALSE,
              file = paste(directory, data.name, ".raw", sep = ""))
  
  sim.name <- paste(title)
  
  filename <- create.file.for.bayesx(directory = directory,
                                     sim.name = sim.name,
                                     data.name = data.name,
                                     distr.name = "normal2",
                                     equation.names = c("sigma", "mu"),
                                     response.name = "y",
                                     form.names = c("const+x1(pspline)", "const+x1(pspline)+x2(random,redist=snmh,priorskew=0.5)"),
                                     seed = r + seed.offset,
                                     iterations = 60000,
                                     burnin = 10000,
                                     step = 10,
                                     predict = TRUE,
                                     getsample = TRUE)
  
  system(paste("~/bsx8/./bayesx", filename))
  #system(paste("C:/fbsx_ohne_bivst/bin/Debug/bayesx.exe", filename))
  
  dat.mu.lin <- read.table("sim_mu_MAIN_mu_REGRESSION_y_LinearEffects.res", header = TRUE)
  dat.mu.x1 <- read.table("sim_mu_MAIN_mu_REGRESSION_y_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.mu.x1.p <- read.table("sim_mu_MAIN_mu_REGRESSION_y_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.mu.x1.v <- read.table("sim_mu_MAIN_mu_REGRESSION_y_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.mu.x2 <- read.table("sim_mu_MAIN_mu_REGRESSION_y_random_effect_of_x2.res", header = TRUE)
  dat.mu.x2.v <- read.table("sim_mu_MAIN_mu_REGRESSION_y_variance_of_random_effect_of_x2.res", header = TRUE)
  dat.mu.x2.s <- read.table("sim_mu_MAIN_mu_REGRESSION_y_skewness_of_random_effect_of_x2.res", header = TRUE)
  dat.mu.x2.o <- read.table("sim_mu_MAIN_mu_REGRESSION_y_random_effect_of_x2_omega.res", header = TRUE)
  dat.mu.x2.a <- read.table("sim_mu_MAIN_mu_REGRESSION_y_random_effect_of_x2_alpha.res", header = TRUE)
  dat.si.lin <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_LinearEffects.res", header = TRUE)
  dat.si.x1 <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  dat.si.x1.p <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_nonlinear_pspline_effect_of_x1_param.res", header = TRUE)
  dat.si.x1.v <- read.table("sim_sigma_MAIN_sigma_REGRESSION_y_variance_of_nonlinear_pspline_effect_of_x1.res", header = TRUE)
  pred <- read.table("sim_mu_MAIN_mu_REGRESSION_y_predict.res", header = TRUE)
  pred.dic <- read.table("sim_mu_MAIN_mu_REGRESSION_y_predict_DIC.res", header = TRUE)
  pred.waic <- read.table("sim_mu_MAIN_mu_REGRESSION_y_predict_WAIC.res", header = TRUE)
  
  save(y,
       x1,
       x2,
       eff.x2,
       dat.mu.lin,
       dat.mu.x1,
       dat.mu.x1.p,
       dat.mu.x1.v,
       dat.mu.x2,
       dat.mu.x2.v,
       dat.mu.x2.s,
       dat.mu.x2.o,
       dat.mu.x2.a,
       dat.si.lin,
       dat.si.x1,
       dat.si.x1.p,
       dat.si.x1.v,
       pred,
       pred.dic,
       pred.waic,
       file = paste(title, r + seed.offset, ".RData", sep = ""))
}