/* BayesX - Software for Bayesian Inference in
Structured Additive Regression Models.
Copyright (C) 2011  Christiane Belitz, Andreas Brezger,
Thomas Kneib, Stefan Lang, Nikolaus Umlauf

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */



#include "FC_hrandom_variance.h"
// added
#include "gsl/gsl_cdf.h"
#include "gsl/gsl_randist.h"

//------------------------------------------------------------------------------
//--------- CLASS: FC_hrandom_variance implementation of member functions ------
//------------------------------------------------------------------------------


namespace MCMC
{


void FC_hrandom_variance::read_options(vector<ST::string> & op,
vector<ST::string> & vn)
  {

  int f;

  /*
  1       degree
  2       numberknots
  3       difforder
  4       lambda
  5       a
  6       b
  7       center
  8       map
  9       lambda_re
  10      a_re
  11      b_re
  12      internal_mult
  */

  FC_nonp_variance::read_options(op,vn);

  if (op[12] == "true")
    {
    mult =true;
    f = op[9].strtodouble(lambdastart);
    f = op[10].strtodouble(a_invgamma);
    f = op[11].strtodouble(b_invgamma);
    }
  else
    {
    mult = false;
    }

  if (op[38] == "true")
    {
    lambdaconst = true;
    nosamples =true;
    }

  // added
  redist = op[80];

  f = op[81].strtodouble(priorvar);
  priorvar = 1 / priorvar;
  if (priorvar < 0) {
    priorvar = 0.001;
  }

  f = op[82].strtodouble(priordf);
  f = op[83].strtodouble(priorskew);
  f = op[84].strtodouble(priordfmean);
  f = op[85].strtodouble(priorvarmean);

  }


FC_hrandom_variance::FC_hrandom_variance(void)
  {

  }


FC_hrandom_variance::FC_hrandom_variance(MASTER_OBJ * mp,unsigned & enr,
                 GENERAL_OPTIONS * o,DISTR * lp,
                  DISTR * lpRE,
                 const ST::string & t,const ST::string & fp,
                 DESIGN * Dp,FC_nonp * FCn,vector<ST::string> & op,
                 vector<ST::string> & vn)
     : FC_nonp_variance(mp,enr,o,lp,t,fp,Dp,FCn,op,vn)
  {
    // added
    redist = "normal";
    priorvar = 100000.0;
    priorskew = 100000.0;
    priordf = 100000.0;
    FChrandom = (FC_hrandom*)FCn;
    psi = 0.0;
    tau = 1.0;
    xi = 0.0;
    df = 30.0;
    omega = 1.0;
    b = sqrt(2 / PI);
    lower = - b;
    FComega = FC(o,"",beta.rows(),1,fp);
    FCalpha = FC(o,"",beta.rows(),1,fp);
    FCskew = FC(o,"",beta.rows(),1,fp);
    FCkurt = FC(o,"",beta.rows(),1,fp);
    FCdf = FC(o,"",beta.rows(),1,fp);
    zs = datamatrix(FCn->beta.rows(), 1, 0);
    ws = datamatrix(FCn->beta.rows(), 1, 1.0);

  read_options(op,vn);
  simplerandom=false;
  likepRE = lpRE;

    // added
    STOREDVALUES::get_st_ef_xi_values(efxi);
    STOREDVALUES::get_st_ef_omega_values(efom);
    STOREDVALUES::get_st_ef_alpha_values(efal);
    STOREDVALUES::get_st_ef_df_values(efdf);
    STOREDVALUES::get_st_iota_values(io);
  }


FC_hrandom_variance::FC_hrandom_variance(MASTER_OBJ * mp,unsigned & enr,
                 GENERAL_OPTIONS * o,DISTR * lp,
                 const ST::string & t,const ST::string & fp,
                 DESIGN * Dp,FC_nonp * FCn,vector<ST::string> & op,
                 vector<ST::string> & vn)
     : FC_nonp_variance(mp,enr,o,lp,t,fp,Dp,FCn,op,vn)
  {
    // added
    redist = "normal";
    priorvar = 100000.0;
    priorskew = 100000.0;
    priordf = 100000.0;
    FChrandom = (FC_hrandom*)FCn;
    psi = 0.0;
    tau = 1.0;
    xi = 0.0;
    df = 30.0;
    omega = 1.0;
    b = sqrt(2 / PI);
    lower = - b;
    FComega = FC(o,"",beta.rows(),1,fp);
    FCalpha = FC(o,"",beta.rows(),1,fp);
    FCskew = FC(o,"",beta.rows(),1,fp);
    FCkurt = FC(o,"",beta.rows(),1,fp);
    FCdf = FC(o,"",beta.rows(),1,fp);
    zs = datamatrix(FCn->beta.rows(), 1, 0);
    ws = datamatrix(FCn->beta.rows(), 1, 1.0);

  read_options(op,vn);
  simplerandom=true;
  simplerandom_linpred=datamatrix(FCn->beta.rows(),1,0);

    // added
    STOREDVALUES::get_st_ef_xi_values(efxi);
    STOREDVALUES::get_st_ef_omega_values(efom);
    STOREDVALUES::get_st_ef_alpha_values(efal);
    STOREDVALUES::get_st_ef_df_values(efdf);
    STOREDVALUES::get_st_iota_values(io);
  }



FC_hrandom_variance::FC_hrandom_variance(const FC_hrandom_variance & m)
  : FC_nonp_variance(FC_nonp_variance(m))
  {
  likepRE = m.likepRE;
  simplerandom=m.simplerandom;
  simplerandom_linpred=m.simplerandom_linpred;
  mult = m.mult;

  // added
  redist = m.redist;
  priorvar = m.priorvar;
  priorskew = m.priorskew;
  priordf = m.priordf;
  FChrandom = m.FChrandom;
  psi = m.psi;
  tau = m.tau;
  xi = m.xi;
  df = m.df;
  omega = m.omega;
  b = m.b;
  lower = m.lower;
  FComega = m.FComega;
  FCalpha = m.FCalpha;
  FCskew = m.FCskew;
  FCkurt = m.FCkurt;
  FCdf = m.FCdf;
  zs = m.zs;
  ws = m.zs;
  STOREDVALUES::get_st_ef_xi_values(efxi);
  STOREDVALUES::get_st_ef_omega_values(efom);
  STOREDVALUES::get_st_ef_alpha_values(efal);
  STOREDVALUES::get_st_ef_df_values(efdf);
  STOREDVALUES::get_st_iota_values(io);
  }


const FC_hrandom_variance & FC_hrandom_variance::operator=(
const FC_hrandom_variance & m)
  {

  if (this==&m)
	 return *this;
  FC_nonp_variance::operator=(FC_nonp_variance(m));
  likepRE = m.likepRE;
  simplerandom=m.simplerandom;
  simplerandom_linpred=m.simplerandom_linpred;
  mult = m.mult;

  // added
  redist = m.redist;
  priorvar = m.priorvar;
  priorskew = m.priorskew;
  priordf = m.priordf;
  FChrandom = m.FChrandom;
  psi = m.psi;
  tau = m.tau;
  xi = m.xi;
  df = m.df;
  omega = m.omega;
  b = m.b;
  lower = m.lower;
  FComega = m.FComega;
  FCalpha = m.FCalpha;
  FCskew = m.FCskew;
  FCkurt = m.FCkurt;
  FCdf = m.FCdf;
  zs = m.zs;
  ws = m.zs;
  STOREDVALUES::get_st_ef_xi_values(efxi);
  STOREDVALUES::get_st_ef_omega_values(efom);
  STOREDVALUES::get_st_ef_alpha_values(efal);
  STOREDVALUES::get_st_ef_df_values(efdf);
  STOREDVALUES::get_st_iota_values(io);

  return *this;
  }


double FC_hrandom_variance::compute_quadform(void)
  {

  unsigned n;
  double sum = 0;
  double * workbeta = FCnonpp->beta.getV();
  register unsigned i;

  n = FCnonpp->beta.rows();


  double * linpredREp;

  if (simplerandom==true)
    linpredREp = simplerandom_linpred.getV();
  else
    {
    if (likepRE->linpred_current==1)
      linpredREp = likepRE->linearpred1.getV();
    else
      linpredREp = likepRE->linearpred2.getV();
    }


  for(i=0;i<n;i++,workbeta++,linpredREp++)
    {
    sum += pow(*workbeta-(*linpredREp),2);
    }

  return sum;

  }


void FC_hrandom_variance::update(void)
  {

  b_invgamma = masterp->level1_likep[equationnr]->trmult*b_invgamma_orig;

  if (lambdaconst==false)
    {
      // change
      if (redist == "sn") {
        double * betap = FCnonpp->beta.getV();
        double * zp = zs.getV();

        double mean = 0.0;
        double v = 1.0;

        double tmp = 0.0;
        double tmp2 = 0.0;
        double tmp3 = 0.0;

        double * linpredREp;

        if (simplerandom==true)
          linpredREp = simplerandom_linpred.getV();
        else {
          if (likepRE->linpred_current==1)
            linpredREp = likepRE->linearpred1.getV();
          else
            linpredREp = likepRE->linearpred2.getV();
        }


        // calculation of the new values of z
        v = 1 / (1 + tau * psi * psi);

        for (int i = 0; i < FCnonpp->beta.rows(); i++, zp++, betap++, linpredREp++){
          mean = v * (tau * psi * ((*betap) - (*linpredREp)) - b);
          *zp = randnumbers::trunc_normal4(lower, mean, sqrt(v));
          tmp += pow((*zp), 2);
          tmp2 += (*zp) * ((*betap) - (*linpredREp));
          tmp3 += pow( (((*betap) - (*linpredREp))) - psi * (*zp),2);
        }

        //calculation of tau
        tau = rand_gamma(a_invgamma+0.5*designp->rankK, b_invgamma+0.5*tmp3);

        // calculation of psi
        v = 1 / (tau * (tmp + priorskew));
        mean = v * (tau * tmp2);
        psi = mean + sqrt(v) * randnumbers::rand_normal();

        // calculate "xi", omega and alpha
        FComega.beta(0,0) = sqrt(1 / tau + psi * psi);
        FCalpha.beta(0,0) = psi * sqrt(tau);
        double delta = FCalpha.beta(0,0) / sqrt(1 + pow(FCalpha.beta(0,0), 2));
        xi = b * FComega.beta(0,0) * delta;
        FCskew.beta(0,0) = 0.5 * (4 - PI) * pow(delta * b / sqrt(1 - pow(b * delta, 2)), 3);
        FComega.acceptance++;
        FCalpha.acceptance++;
        FCskew.acceptance++;

        FChrandom->omega = FComega.beta(0,0);
        FChrandom->alpha = FCalpha.beta(0,0);
        FChrandom->xi = xi;

        FComega.update();
        FCalpha.update();
        FCskew.update();

        beta(0,0) = pow(FComega.beta(0,0),2) * (1 - 2 / PI * pow(delta, 2));
      }
      else if (redist == "snmh") {
        double * betap = FCnonpp->beta.getV();

        double avg = 0.0;

        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          avg += (*betap);
        }
        avg = avg / FCnonpp->beta.rows();

        betap = FCnonpp->beta.getV();

        double b2 = pow(b, 2);

        // calculation of the sd stored in tau
        double s = sqrt(tau);
        double g = psi; //0.9952716*((*worklin[0])/(sqrt(1 + (*worklin[0])*(*worklin[0]))));
        double R = cbrt(2*g/(4-PI));
        double al = R/sqrt(b2-(1-b2)*R*R);
        double d = al/sqrt(1 + al*al);
        double d2 = pow(d, 2);
        double om = s/sqrt(1 - b2*d2);
        xi = b*om*d;

        double v = 0.0;
        double z = 0.0;
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / om;
          v += (z * z - b * d * z - 1 - al * randnumbers::phi(al*z) / randnumbers::Phi2(al*z) * (z - b * d));
        }

        double w = b2 * d2 * (1 + al * al * supportfunctions::sn_a0(al) - 2 * (1 + 2 * al * al) / (1 + al * al)) - 2 * b * d * al * al * supportfunctions::sn_a1(al) + 2 + al * al * supportfunctions::sn_a2(al);
        double P = w * FCnonpp->beta.rows();
        double mu = log(s) + v / P;
        double s_s = exp(mu + randnumbers::rand_normal() / sqrt(P));

        double om_s= s_s / sqrt(1 - b2 * d2);
        double xi_s = b*om_s*d;

        v = 0.0;
        double z_s = 0.0;
        double acc = FCnonpp->beta.rows() * (log(om) - log(om_s));

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / om;
          z_s = ((*betap) + xi_s - avg) / om_s;
          v += (z_s * z_s - b * d * z_s - 1 - al * randnumbers::phi(al*z_s) / randnumbers::Phi2(al*z_s) * (z - b * d));
          acc += - 0.5 * (pow(z_s, 2) - pow(z,2)) + log(randnumbers::Phi2(al*z_s)) - log(randnumbers::Phi2(al*z));
        }

        double mu_s = log(s_s) + v / P;

        acc += 0.5 * pow(log(s_s) - mu, 2) * P - 0.5 * pow(log(s) - mu_s, 2) * P - 0.5 * log(P) + 0.5 * log(P);

        //acc += (- 0.5 * pow(log(s_s) - priorvarmean, 2) + 0.5 * pow(log(s) - priorvarmean, 2)) / priorvar;
        acc += - (a_invgamma - 1.0) * (pow(s_s, 2) - pow(s, 2)) - b_invgamma / pow(s_s,2) + b_invgamma / pow(s, 2);

        if (log(randnumbers::uniform()) < acc) {
          tau = pow(s_s, 2);
          s = s_s;
          om = om_s;
          xi = xi_s;
        }
        else {
          acceptance--;
        }

        // calculation of the skewness stored in psi

  //      double g1 = g;
  //      if(abs(g) < 0.001){
  //        g = 0.001;
  //      }
  //      R = cbrt(2*g/(4-PI));
  //      al = R/sqrt(b2-(1-b2)*R*R);
  //      d = al/sqrt(1 + al*al);
  //      d2 = pow(d, 2);
  //
  //      double R1 = cbrt(2*g1/(4-PI));
  //      double al1 = R1/sqrt(b2-(1-b2)*R1*R1);
  //      double d1 = al1 / sqrt(1 + al1*al1);
  //      double om1 = s/sqrt(1 - b2 * d1 * d1);
  //      double xi1 = b * om1 * d1;
  //
  //      double e_g = g / sqrt(0.9952716 * 0.9952716 - g * g); // eta gamma
  //
  //      double zt1 = 0.0;
  //      double a0 = supportfunctions::sn_a0(al);
  //      double a1 = supportfunctions::sn_a1(al);
  //      double a2 = supportfunctions::sn_a2(al);
  //
  //      v = 0.0;
  //      P = 0.0;
  //
  //      betap = FCnonpp->beta.getV();
  //      for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
  //        z = ((*betap) + xi - avg) / om;
  //        zt1 = randnumbers::phi(al*z) / randnumbers::Phi2(al*z);
  //        v += (((al * b * zt1 - z * b + b2 * d / (1 - b2 * d2) * (al * (b * d - z) * zt1 - z * b * d - 1 + z * z))
  //            / pow(1 + al * al, 1.5) + z * zt1)
  //           * (1 / sqrt(b2 - (1 - b2) * R * R) + (1 - b2) * R * R / pow(sqrt(b2 - (1 - b2) * R * R),3)) * cbrt(2 / (4 - PI) / pow(g,2)) / 3
  //           * 0.9952716 / pow(sqrt(1 + e_g * e_g), 3));
  //      }
  //      w = ((a2 + 2 / pow(sqrt(1 + al * al), 3) * (b * al * a1 + (b * d * al * a1 - al * a2) * b2 * d / (1 - b2 * d2))
  //          + (b2 * (al * al * a0 - 1) - 2 * b2 * b * d / (1 - b2 * d2) * (b * d2 * d + al * al * a1 + b * d * (1 - al * al * a0))
  //             + b2 * b2 * d2 / pow(1 - b2 * d2, 2) * (2 - b2 * d2 * (1 + 2 * d2) + al * al * (a2 - 2 * b * d * a1 + b2 * d2 * a0)))
  //          / pow(1 + al * al, 3))
  //         * pow((1 / sqrt(b2 - (1 - b2) * R * R) + (1 - b2) * R * R / pow(sqrt(b2 - (1 - b2) * R * R),3)) * cbrt(2 / (4 - PI) / pow(g,2)) / 3
  //               * 0.9952716 / pow(sqrt(1 + e_g * e_g), 3), 2));
  //      P = w * FCnonpp->beta.rows();
  //      mu = e_g + v / P;
  //      double e_g_s = mu + randnumbers::rand_normal() / sqrt(P);
  //      double g_s = 0.9952716*((e_g_s)/(sqrt(1 + (e_g_s)*(e_g_s))));
  //
  //      double g1_s = g_s;
  //      if(abs(g_s) < 0.001) {
  //        g_s = 0.001;
  //      }
  //
  //      double R_s = cbrt(2*g_s/(4-PI));
  //      double al_s = R_s/sqrt(b2-(1-b2)*R_s*R_s);
  //      double d_s = al_s/sqrt(1 + al_s*al_s);
  //      double d2_s = pow(d_s, 2);
  //
  //      double R1_s = cbrt(2*g1_s/(4-PI));
  //      double al1_s = R1_s/sqrt(b2-(1-b2)*R1_s*R1_s);
  //      double d1_s = al1_s/sqrt(1 + al1_s*al1_s);
  //      double om1_s = s / sqrt(1 - b2 * d1_s * d1_s);
  //      double xi1_s = b * om1_s * d1_s;
  //
  //      om_s = s / sqrt(1 - b2 * d2_s);
  //      xi_s = b*om_s*d_s;
  //
  //      a0 = supportfunctions::sn_a0(al_s);
  //      a1 = supportfunctions::sn_a1(al_s);
  //      a2 = supportfunctions::sn_a2(al_s);
  //
  //      acc = 0.0;
  //      v = 0.0;
  //      double z1 = 0.0;
  //      double z1_s = 0.0;
  //
  //      betap = FCnonpp->beta.getV();
  //      for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
  //        z = ((*betap) + xi - avg) / om;
  //        z_s = ((*betap) + xi_s - avg) / om_s;
  //        z1 = ((*betap) + xi1 - avg) / om1;
  //        z1_s = ((*betap) + xi1_s - avg) / om1_s;
  //        zt1 = randnumbers::phi(al_s*z_s) / randnumbers::Phi2(al_s*z_s); // hier
  //        v += (((al_s * b * zt1 - z_s * b + b2 * d_s / (1 - b2 * d2_s) * (al_s * (b * d_s - z_s) * zt1 - z_s * b * d_s - 1 + z_s * z_s))
  //            / pow(1 + al_s * al_s, 1.5) + z_s * zt1)
  //           * (1 / sqrt(b2 - (1 - b2) * R_s * R_s) + (1 - b2) * R_s * R_s / pow(sqrt(b2 - (1 - b2) * R_s * R_s),3)) * cbrt(2 / (4 - PI) / pow(g_s,2)) / 3
  //           * 0.9952716 / pow(sqrt(1 + e_g_s  * e_g_s), 3));
  //        acc += - log(om1_s) - 0.5*pow(z1_s, 2) + log(om1) + 0.5*pow(z1, 2) + log(randnumbers::Phi2(al1_s*z1_s)) - log(randnumbers::Phi2(al1*z1));
  //      }
  //      w = ((a2 + 2 / pow(sqrt(1 + al_s * al_s), 3) * (b * al_s * a1 + (b * d_s * al_s * a1 - al_s * a2) * b2 * d_s / (1 - b2 * d2_s))
  //          + (b2 * (al_s * al_s * a0 - 1) - 2 * b2 * b * d_s / (1 - b2 * d2_s) * (b * d2_s * d_s + al_s * al_s * a1 + b * d_s * (1 - al_s * al_s * a0))
  //             + b2 * b2 * d2_s / pow(1 - b2 * d2_s, 2) * (2 - b2 * d2_s * (1 + 2 * d2_s) + al_s * al_s * (a2 - 2 * b * d_s * a1 + b2 * d2_s * a0)))
  //          / pow(1 + al_s * al_s, 3))
  //         * pow((1 / sqrt(b2 - (1 - b2) * R_s * R_s) + (1 - b2) * R_s * R_s / pow(sqrt(b2 - (1 - b2) * R_s * R_s),3)) * cbrt(2 / (4 - PI) / pow(g_s,2)) / 3
  //               * 0.9952716 / pow(sqrt(1 + e_g_s  * e_g_s), 3), 2));
  //
  //      double P_s = FCnonpp->beta.rows() * w;
  //
  //      mu_s = e_g_s + v / P_s;
  //
  //      acc += 0.5 * pow(e_g_s  - mu, 2) * P - 0.5 * pow(e_g - mu_s, 2) * P_s + 0.5 * log(P_s) - 0.5 * log(P);
  //
  ////      acc += (- 0.5 * pow(e_g_s, 2) + 0.5 * pow(e_g, 2)) / priorskew;
  //
  //      acc += - log(pow(priorskew, 2) + pow(e_g_s, 2)) + log(pow(priorskew, 2) + pow(e_g, 2)); // Cauchy Prior
  //
  //      if (abs(g_s) < 0.99) {
  //        if ( acc > log(randnumbers::uniform())) {
  //          psi = g1_s;
  //          om = om1_s;
  //          d = d1_s;
  //          al = al1_s;
  //          xi = xi1_s;
  //          FCskew.acceptance++;
  //        }
  //        else {
  //          om = om1;
  //          al = al1;
  //          xi = xi1;
  //        }
  //      }
  //      else {
  //        om = om1;
  //        al = al1;
  //        xi = xi1;
  //      }

        psi = 0.0;
        al = 0.0;

        // calculate "xi", omega and alpha
        FCskew.beta(0,0) = psi;
        FCalpha.beta(0,0) = al;
        FComega.beta(0,0) = om;

        FChrandom->omega = om;
        FChrandom->alpha = al;
        FChrandom->xi = xi;

        FCskew.update();
        FCalpha.update();
        FComega.update();

        beta(0,0) = tau;
      }
      else if (redist == "t") {
        // mod gibbs

        double * betap = FCnonpp->beta.getV();
        double * zp = zs.getV();
        double * wp = ws.getV();

        double mean = 0.0;
        double sd = 1.0;

        double tmp3 = 0.0; // store (yw )^T (yw) in tau

        double * linpredREp;

        if (simplerandom==true)
          linpredREp = simplerandom_linpred.getV();
        else {
          if (likepRE->linpred_current==1)
            linpredREp = likepRE->linearpred1.getV();
          else
            linpredREp = likepRE->linearpred2.getV();
        }

        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++, linpredREp++, wp++){
          if (*wp == 0.0) {
            *wp = 1.0; // initialize values, should not be zero later on (~GAMMA)
          }
          tmp3 += pow((*wp) * (((*betap) - (*linpredREp))),2);
        }

        //calculation of tau
        tau = rand_gamma(a_invgamma+0.5*designp->rankK, b_invgamma+0.5*tmp3);

        // calculation of df
        double v = 0.0;
        double dfhalfplusone = 0.5 * (df + 1.0);
        double dfhalf = 0.5 * df;
        double zob = 0.0;
        double avg = 0.0;

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          zob = ((*betap) - avg) / omega;
          v += 0.5 * df * (randnumbers::digamma_exact(dfhalfplusone) - randnumbers::digamma_exact(dfhalf) - log(1 + pow(zob, 2) / df)) - 0.5 + 0.5 * (df + 1.0) * pow((*betap), 2) / (pow(omega, 2) * df + pow((*betap), 2));
        }

        double w = - 0.25 * pow(df,2) * (randnumbers::trigamma_exact(dfhalfplusone) - randnumbers::trigamma_exact(dfhalf)) + 0.5 * df / (df + 3.0) - df / (df + 1.0);
        double P = FCnonpp->beta.rows() * w;
        double mu = log(df) + v / P;
        double dfs = exp(mu + randnumbers::rand_normal() / sqrt(P));

        double vs = 0.0;
        double dfshalfplusone = 0.5 * (dfs + 1.0);
        double dfshalf = 0.5 * dfs;

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          zob = ((*betap) - avg) / omega;
          vs += 0.5 * dfs * (randnumbers::digamma_exact(dfshalfplusone) - randnumbers::digamma_exact(dfshalf) - log(1 + pow(zob, 2) / dfs)) - 0.5 + 0.5 * (dfs + 1.0) * pow((*betap), 2) / (pow(omega, 2) * dfs + pow((*betap), 2));
        }

        double w_s = - 0.25 * pow(dfs,2) * (randnumbers::trigamma_exact(dfshalfplusone) - randnumbers::trigamma_exact(dfshalf)) + 0.5 * dfs / (dfs + 3.0) - dfs / (dfs + 1.0);
        double P_s = FCnonpp->beta.rows() * w_s;
        double mus = log(dfs) + v / P;

        betap = FCnonpp->beta.getV();
        double acc = 0.0;
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          zob = ((*betap) - avg) / omega;
          acc += (- 0.5 * log(dfs) + randnumbers::lngamma_exact(dfshalfplusone) - randnumbers::lngamma_exact(dfshalf) - 0.5 * (dfs + 1.0) * log(1 + pow(zob,2) / dfs)
                  + 0.5 * log(df) - randnumbers::lngamma_exact(dfhalfplusone) + randnumbers::lngamma_exact(dfhalf) + 0.5 * (df + 1.0) * log(1 + pow(zob, 2) / df));
        }
  //      cout << mu << " " << mus << " " << dfs << " " << exp(acc);
        acc += 0.5 * pow(log(dfs) - mu, 2) * P - 0.5 * pow(log(df) - mus, 2) * P_s + 0.5 * log(P_s) - 0.5 * log(P);

        acc += (- 0.5 * pow(log(dfs) - priordfmean, 2) + 0.5 * pow(log(df) - priordfmean, 2)) / priordf;
  // cout << " " << exp(acc) << endl;
        if (log(randnumbers::uniform()) < acc & dfs > 4 & dfs < 200) {
          df = dfs;
          FCdf.acceptance++;
          FCkurt.acceptance++;
  //        cout << "accepted" << endl;
        }

        // draw new values for w
        betap = FCnonpp->beta.getV();
        if (simplerandom==true)
          linpredREp = simplerandom_linpred.getV();
        else {
          if (likepRE->linpred_current==1)
            linpredREp = likepRE->linearpred1.getV();
          else
            linpredREp = likepRE->linearpred2.getV();
        }
        wp = ws.getV();
        for(int i = 0; i < FCnonpp->beta.rows(); i++, betap++, wp++, linpredREp++) {
          (*wp) = sqrt(randnumbers::rand_gamma(0.5 * (df + 1.0), 0.5 * (df + pow((((*betap) - (*linpredREp))) / tau, 2))));
        }

        // calculate "xi", omega and alpha
        FCdf.beta(0,0) = df;
        FComega.beta(0,0) = sqrt(1 / tau);
        double ku = 0.0;
        if (df > 4.0) {
          ku = (3.0 * df - 6) / (df - 4.0);
        }
        FCkurt.beta(0,0) = ku;
        FCkurt.acceptance++;
        FComega.acceptance++;
        FCalpha.acceptance++;

        FChrandom->df = df;
        FChrandom->omega = FComega.beta(0,0);

        FComega.update();
        FCdf.update();
        FCkurt.update();

        beta(0,0) = (pow(FComega.beta(0,0),2)) * df / (df - 2.0);
      }
      else if (redist == "tmh") {
        double * betap = FCnonpp->beta.getV();

        double avg = 0.0;

        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          avg += (*betap);
        }
        avg = avg / FCnonpp->beta.rows();

        avg = 0.0;

        betap = FCnonpp->beta.getV();

        double * linpredREp;

        if (simplerandom==true)
          linpredREp = simplerandom_linpred.getV();
        else {
          if (likepRE->linpred_current==1)
            linpredREp = likepRE->linearpred1.getV();
          else
            linpredREp = likepRE->linearpred2.getV();
        }

        //calculation of omega
        double v = 0.0;
        for(int i = 0; i < FCnonpp->beta.rows(); i++, betap++){
          v += -0.5 + 0.5 * (df + 1.0) * pow((*betap) - avg,2) / (pow(omega, 2) * df + pow((*betap - avg), 2));
        }
        double w = 0.5 * df / (df + 3.0);
        double P = FCnonpp->beta.rows() * w;
        double mu = log(omega) + v / P;
        double omegas = exp(mu + randnumbers::rand_normal() / sqrt(P));

        betap = FCnonpp->beta.getV();
        double vs = 0.0;
        for(int i = 0; i < FCnonpp->beta.rows(); i++, betap++){
          vs += -0.5 + 0.5 * (df + 1.0) * pow((*betap) - avg,2) / (pow(omegas, 2) * df + pow((*betap) - avg, 2));
        }

        double mus = log(omega) + vs / P;

        betap = FCnonpp->beta.getV();
        double acc = 0.0;
        double zob = 0.0;
        double zobs = 0.0;
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++, linpredREp++) {
          zob = ((*betap) - avg) / omega;
          zobs = ((*betap) - avg) / omegas;
          acc += (- log(omegas) - 0.5 * (df + 1.0) * log(1 + pow(zobs, 2) / df)
                  + log(omega) + 0.5 * (df + 1.0) * log(1 + pow(zob, 2) / df));
        }
        acc += 0.5 * pow(log(omegas) - mu, 2) * P - 0.5 * pow(log(omega) - mus, 2) * P;// + 0.5 * log(P_s) - 0.5 * log(P);

        if (log(randnumbers::uniform()) < acc) {
          omega = omegas;
          FComega.acceptance++;
        }

        // calculation of df
        betap = FCnonpp->beta.getV();
        v = 0.0;
        double dfhalfplusone = 0.5 * (df + 1.0);
        double dfhalf = 0.5 * df;
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          zob = ((*betap) - avg) / omega;
          v += 0.5 * df * (randnumbers::digamma_exact(dfhalfplusone) - randnumbers::digamma_exact(dfhalf) - log(1 + pow(zob, 2) / df)) - 0.5 + 0.5 * (df + 1.0) * pow((*betap), 2) / (pow(omega, 2) * df + pow((*betap), 2));
        }

        w = - 0.25 * pow(df,2) * (randnumbers::trigamma_exact(dfhalfplusone) - randnumbers::trigamma_exact(dfhalf)) + 0.5 * df / (df + 3.0) - df / (df + 1.0);
        P = FCnonpp->beta.rows() * w;
        mu = log(df) + v / P;
        double dfs = exp(mu + randnumbers::rand_normal() / sqrt(P));

        betap = FCnonpp->beta.getV();
        vs = 0.0;
        double dfshalfplusone = 0.5 * (dfs + 1.0);
        double dfshalf = 0.5 * dfs;
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          zob = ((*betap) - avg) / omega;
          vs += 0.5 * dfs * (randnumbers::digamma_exact(dfshalfplusone) - randnumbers::digamma_exact(dfshalf) - log(1 + pow(zob, 2) / dfs)) - 0.5 + 0.5 * (dfs + 1.0) * pow((*betap), 2) / (pow(omega, 2) * dfs + pow((*betap), 2));
        }

        double ws = - 0.25 * pow(dfs,2) * (randnumbers::trigamma_exact(dfshalfplusone) - randnumbers::trigamma_exact(dfshalf)) + 0.5 * dfs / (dfs + 3.0) - dfs / (dfs + 1.0);
        double P_s = FCnonpp->beta.rows() * ws;
        mus = log(dfs) + v / P;

        betap = FCnonpp->beta.getV();
        acc = 0.0;
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          zob = ((*betap) - avg) / omega;
          acc += (- 0.5 * log(dfs) + randnumbers::lngamma_exact(dfshalfplusone) - randnumbers::lngamma_exact(dfshalf) - 0.5 * (dfs + 1.0) * log(1 + pow(zob,2) / dfs)
                  + 0.5 * log(df) - randnumbers::lngamma_exact(dfhalfplusone) + randnumbers::lngamma_exact(dfhalf) + 0.5 * (df + 1.0) * log(1 + pow(zob, 2) / df));
        }
  //      cout << mu << " " << mus << " " << dfs << " " << exp(acc);
        acc += 0.5 * pow(log(dfs) - mu, 2) * P - 0.5 * pow(log(df) - mus, 2) * P_s + 0.5 * log(P_s) - 0.5 * log(P);

        acc += (- 0.5 * pow(log(dfs) - priordfmean, 2) + 0.5 * pow(log(df) - priordfmean, 2)) / priordf;
  // cout << " " << exp(acc) << endl;
        if (log(randnumbers::uniform()) < acc & dfs > 4 & dfs < 200) {
          df = dfs;
          FCdf.acceptance++;
          FCkurt.acceptance++;
  //        cout << "accepted" << endl;
        }

        // calculate "xi", omega and alpha
        FCdf.beta(0,0) = df;
        FComega.beta(0,0) = omega;
        double ku = 0.0;
        if (df > 4.0) {
          ku = (3.0 * df - 6) / (df - 4.0);
        }
        FCkurt.beta(0,0) = ku;

        FChrandom->df = df;
        FChrandom->omega = omega;

        FComega.update();
        FCdf.update();
        FCkurt.update();

        beta(0,0) = (pow(omega,2)) * df / (df - 2.0);
      }
      else if (redist == "st") {
        double * betap = FCnonpp->beta.getV();
        double * zp = zs.getV();
        double * wp = ws.getV();

        double d = 10.0;

        double avg = 0.0;

        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          avg += (*betap);
        }
        avg = avg / FCnonpp->beta.rows();

        avg = 0.0;

        betap = FCnonpp->beta.getV();

        double mean = 0.0;
        double sd = 1.0;

        double tmp = 0.0; //store the sum for Xw^T Xw in v_beta
        double tmp2 = 0.0; // store the sum for Xw^T yw in m_beta
        double tmp3 = 0.0; // store (yw - Xw psi)^T (yw - Xw psi) in tau

        double * linpredREp;

        if (simplerandom==true)
          linpredREp = simplerandom_linpred.getV();
        else {
          if (likepRE->linpred_current==1)
            linpredREp = likepRE->linearpred1.getV();
          else
            linpredREp = likepRE->linearpred2.getV();
        }

        // calculation of the new values of z
        double v = 1 / (1 + tau * psi * psi);

        for (int i = 0; i < FCnonpp->beta.rows(); i++, zp++, betap++, linpredREp++, wp++){
          if (*wp == 0.0) {
            *wp = 1.0; // initialize values, should not be zero later on (~GAMMA)
          }
          mean = v * (tau * psi * ((*betap) - (*linpredREp)) - b);
          *zp = randnumbers::trunc_normal4(lower, mean, sqrt(v) / (*wp));
          tmp += pow((*zp) * (*wp), 2);
          tmp2 += (*zp) * (*wp) * (*wp) * ((*betap) - (*linpredREp));
          tmp3 += pow((*wp) * (((*betap) - (*linpredREp)) - psi * (*zp)),2);
        }

        //calculation of tau
        tau = rand_gamma(a_invgamma+0.5*designp->rankK, b_invgamma+0.5*tmp3);

        // calculation of psi
        v = 1 / (tau * (tmp + priorvar));
        mean = v * (tau * tmp2);
        psi = mean + sqrt(v) * randnumbers::rand_normal();

        // store omega and alpha
        FComega.beta(0,0) = sqrt(1 / tau + psi * psi);
        FCalpha.beta(0,0) = psi * sqrt(tau);
        double delta = FCalpha.beta(0,0) / sqrt(1 + pow(FCalpha.beta(0,0), 2));


        double dfminusonehalf = 0.5 * (df - 1.0);
        double dfhalf = 0.5 * df;
        double dfplusonehalf = 0.5 * (df + 1.0);
        double dfhalfplusone = 0.5 * df + 1.0;

        double bdf = sqrt(df / PI) * randnumbers::gamma_exact(dfminusonehalf) / randnumbers::gamma_exact(dfhalf);

        xi = sqrt(1 / tau + psi * psi) * delta * bdf;

        v = 0.0;
        double z = 0.0;
        double tval = 0.0;
        double Tval = 0.0;
        double tmpp = 0.0;
        double iota = 0.0;
        double alpha =  psi * sqrt(tau);
        double tau_o = 0.0;

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / sqrt(1 / tau + psi * psi);
          tau_o = sqrt((df+1)/(pow(z,2) + df));
          tmpp = alpha*z*tau_o;
          tval = gsl_ran_tdist_pdf(tmpp, df + 1.0);
          Tval = gsl_cdf_tdist_P(tmpp, df+1.0);
          iota = io[(max(-100, min(100, (int)nearbyint(tmpp*2.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)];

          v = v + (0.5 * (randnumbers::digamma_exact(dfhalfplusone) - randnumbers::digamma_exact(dfhalf) - (2 * df + 1) / (df + 1) / df - log(1.0 + pow(z,2) / df)
                        + pow(z * tau_o, 2) / df + alpha * z * (z * z - 1) / pow(df+z*z, 2) / tau_o * tval / Tval + iota / Tval)
                 - 0.5 * sqrt(1 / tau + psi * psi) * delta * bdf * (randnumbers::digamma_exact(dfminusonehalf) - randnumbers::digamma_exact(dfhalf) + 1.0 / df) * (z * pow(tau_o,2) / sqrt(1 / tau + psi * psi) - alpha * tau_o * df / (pow(z,2) + df) / sqrt(1 / tau + psi * psi) * tval / Tval)) * df;
        }

        double w = (efdf[(max(-200, min(500, (int)nearbyint(log(df)*100.0))) + 200)]
            + pow( 0.5 * sqrt(1 / tau + psi * psi) * delta * bdf * (randnumbers::digamma_exact(dfminusonehalf) - randnumbers::digamma_exact(dfhalf) + 1.0 / df), 2) * efxi[(max(-100, min(100, (int)nearbyint(alpha*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)]) * pow(df, 2);

        double P = w * FCnonpp->beta.rows();

        double mu = log(df) + v / P;

        double df_s = exp(mu + randnumbers::rand_normal() / sqrt(P));

        if (df_s > 100.0) {
          df_s = 100.0;
        }

        double dfsminusonehalf = 0.5 * (df_s - 1.0);
        double dfshalf = 0.5 * df_s;
        double dfsplusonehalf = 0.5 * (df_s + 1.0);
        double dfshalfplusone = 0.5 * df_s + 1.0;

        double bdf_s = sqrt(df_s / PI) * randnumbers::gamma_exact(dfsminusonehalf) / randnumbers::gamma_exact(dfshalf);

        double xi_s = omega * delta * bdf_s;

        v = 0.0;
        double acc = 0.0;
        double tmp_s;
        double tval_s;
        double Tval_s;
        double z_s;
        double tau_s;

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / omega;
          z_s = ((*betap) + xi_s - avg) / omega;
          tau_o = sqrt((df+1)/(pow(z,2) + df));
          tau_s = sqrt((df_s+1)/(pow(z_s,2) + df_s));
          tmp = alpha*z*tau_o;
          tmp_s = alpha*z_s*tau_s;
          tval_s = gsl_ran_tdist_pdf(tmp_s, df_s + 1.0);
          Tval_s = gsl_cdf_tdist_P(tmp_s, df_s+1.0);
          Tval = gsl_cdf_tdist_P(tmp, df+1.0);
          iota = io[(max(-100, min(100, (int)nearbyint(tmp_s*2.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df_s)*10.0))) + 20)];
          v += (0.5 * (randnumbers::digamma_exact(dfshalfplusone) - randnumbers::digamma_exact(dfshalf) - (2 * df_s + 1) / (df_s + 1) / df_s - log(1.0 + pow(z_s,2) / df_s)
                        + pow(z_s * tau_s, 2) / df_s + alpha * z_s * (z_s * z_s - 1) / pow(df_s+z_s*z_s, 2) / tau_s * tval_s / Tval_s + iota / Tval_s)
                - 0.5 * omega * delta * bdf_s * (randnumbers::digamma_exact(dfsminusonehalf) - randnumbers::digamma_exact(dfshalf) + 1.0 / df_s) * (z_s * pow(tau_s,2) / omega - alpha * tau_s * df_s / (pow(z_s,2) + df_s) / omega * tval_s / Tval_s)) * df_s;
          acc += - 0.5*log(df_s) + randnumbers::lngamma_exact(dfsplusonehalf) - randnumbers::lngamma_exact(dfshalf) - 0.5*(df_s+1)*log(1 + pow(z_s,2)/df_s) + log(Tval_s) - (- 0.5*log(df) + randnumbers::lngamma_exact(dfplusonehalf) - randnumbers::lngamma_exact(dfhalf) - 0.5*(df+1)*log(1 + pow(z,2)/df) + log(Tval));
        }

        w = (efdf[(max(-200, min(500, (int)nearbyint(log(df_s)*100.0))) + 200)]
            + pow( 0.5 * omega * delta * bdf_s * (randnumbers::digamma_exact(dfsminusonehalf) - randnumbers::digamma_exact(dfshalf) + 1.0 / df_s), 2) * efxi[(max(-100, min(100, (int)nearbyint(alpha*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df_s)*10.0))) + 20)]) * pow(df_s, 2);

        double P_s = w * FCnonpp->beta.rows();

        double mu_s = log(df_s) + v / P_s;

        acc += 0.5 * pow(log(df_s) - mu, 2) * P - 0.5 * pow(log(df) - mu_s, 2) * P_s + 0.5 * log(P_s) - 0.5 * log(P);

        acc += (- 0.5 * pow(log(df_s) - priordfmean, 2) + 0.5 * pow(log(df) - priordfmean, 2)) / priordf; // prior

        if (log(randnumbers::uniform()) < acc && (df_s > 4.0)) {
          df = df_s;
          xi = xi_s;
          FCdf.acceptance++;
        }



        // draw new values for w
        betap = FCnonpp->beta.getV();
        zp = zs.getV();
        wp = ws.getV();
        for(int i = 0; i < FCnonpp->beta.rows(); i++, betap++, wp++, zp++) {
          (*wp) = sqrt(randnumbers::rand_gamma(0.5 * (df + 1.0), 0.5 * (df + (*zp) * (*zp) + pow((((*betap) - (*linpredREp)) - psi * (*zp)) / tau, 2))));
        }

        // calculate "xi", omega and alpha
        FCdf.beta(0,0) = df;

        double sk = 0.0;
        double ku = 0.0;
        double sigZ = 0.0;
        dfminusonehalf = 0.5 * (df - 1.0);
        dfhalf = 0.5 * df;
        bdf = sqrt(df / PI) * randnumbers::gamma_exact(dfminusonehalf) / randnumbers::gamma_exact(dfhalf);
        sigZ = sqrt(df/(df-2)-pow(bdf*delta,2));
        if(df > 3){
          sk = (df*(3-delta*delta)/(df-3)-3.0*df/(df-2)+2*pow(bdf*delta,2))*bdf*delta/pow(sqrt(sigZ),3);
        }
        if(df > 4){
          ku = (3*pow(df,2)/(df-2)/(df-4)-4*pow(bdf*delta,2)*df*(3-pow(delta,2))/(df-3)+6*pow(bdf*delta,2)*df/(df-2)-3*pow(bdf*delta,4))/pow(sigZ,4);
        }
        FCskew.beta(0,0) = sk;
        FCkurt.beta(0,0) = ku;
        xi = bdf * FComega.beta(0,0) * delta;

        FComega.acceptance++;
        FCalpha.acceptance++;
        FCskew.acceptance++;
        FCkurt.acceptance++;

        FChrandom->df = df;
        FChrandom->omega = FComega.beta(0,0);
        FChrandom->alpha = FCalpha.beta(0,0);
        FChrandom->xi = xi;

        FComega.update();
        FCalpha.update();
        FCdf.update();
        FCskew.update();
        FCkurt.update();

        beta(0,0) = pow(FComega.beta(0,0),2) * (df / (df - 2.0) - pow(bdf * delta, 2));


      }
      else if (redist == "stmh") {
        double * betap = FCnonpp->beta.getV();
        double * zp = zs.getV();
        double * wp = ws.getV();

        double avg = 0.0;

        double d = 10.0;

        double * linpredREp;

        if (simplerandom==true)
          linpredREp = simplerandom_linpred.getV();
        else {
          if (likepRE->linpred_current==1)
            linpredREp = likepRE->linearpred1.getV();
          else
            linpredREp = likepRE->linearpred2.getV();
        }

        double dfhalfplusone = 0.5*df + 1.0;
        double dfhalf = 0.5 * df;
        double dfminusonehalf = 0.5 * (df - 1.0);
        double dfplusonehalf = 0.5 * (df + 1.0);

        double bdf = sqrt(df / PI) * randnumbers::gamma_exact(dfminusonehalf) / randnumbers::gamma_exact(dfhalf);
        double alpha = psi;
        double delta = alpha / sqrt(1 + pow(alpha, 2));
        xi = omega * delta * bdf;

        // calculation of omega
        double v = 0.0;
        double z = 0.0;

        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / omega;
          tau = sqrt((df+1)/(pow(z,2) + df));
          v += ((- 1 + pow(z*tau, 2) - alpha*z*tau*df / (pow(z,2) + df) * gsl_ran_tdist_pdf(alpha*z*tau, df + 1.0) / gsl_cdf_tdist_P(alpha*z*tau, df + 1.0)) / omega
                  - bdf * delta * (z * pow(tau,2) / omega - alpha * tau * df / (pow(z,2) + df) / omega * gsl_ran_tdist_pdf(alpha*z*tau, df + 1) / gsl_cdf_tdist_P(alpha*z*tau, df + 1))) * omega;
        }

        double w = (efom[(max(-100, min(100, (int)nearbyint(alpha*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)]
            + pow(bdf * delta, 2) * efxi[(max(-100, min(100, (int)nearbyint(alpha*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)]);

        double P = w * FCnonpp->beta.rows();

        double mu = log(omega) + v / P;

        double omega_s = exp(mu + randnumbers::rand_normal() / sqrt(P));

        double xi_s = bdf * omega_s * delta;

        v = 0.0;
        double acc = 0.0;

        betap = FCnonpp->beta.getV();
        double z_s = 0.0;
        double tau_s = 0.0;

        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / omega;
          z_s = ((*betap) + xi - avg) / omega_s;
          tau = sqrt((df+1)/(pow(z,2) + df));
          tau_s = sqrt((df+1)/(pow(z_s,2) + df));
          v += ((- 1 + pow(z_s*tau_s, 2) - alpha*z_s*tau_s*df / (pow(z_s,2) + df) * gsl_ran_tdist_pdf(alpha*z_s*tau_s, df + 1.0) / gsl_cdf_tdist_P(alpha*z_s*tau_s, df + 1.0)) / omega_s
                 - bdf * delta * (z_s * pow(tau_s,2) / omega_s - alpha * tau_s * df / (pow(z_s,2) + df) / omega_s * gsl_ran_tdist_pdf(alpha*z_s*tau_s, df + 1) / gsl_cdf_tdist_P(alpha*z_s*tau_s, df + 1))) * omega_s;
          acc += log(omega) - log(omega_s) - 0.5*(df+1)*log(1 + pow(z_s,2)/df) + log(gsl_cdf_tdist_P(alpha*z_s*tau_s, df+1)) + 0.5*(df+1)*log(1 + pow(z,2)/df) - log(gsl_cdf_tdist_P(alpha*z*tau, df+1));;
        }

  //    w = (ef.omega4(0, - xi_s, omega_s, alpha, df) + pow(bdf * delta, 2) * ef.xi4(0, - xi_s, omega_s, alpha, df)) * pow(omega_s, 2);

        double P_s = w * FCnonpp->beta.rows();

        double mu_s = log(omega_s) + v / P_s;

        acc += 0.5 * pow(log(omega_s) - mu, 2) * P - 0.5 * pow(log(omega) - mu_s, 2) * P_s + 0.5 * log(P_s) - 0.5 * log(P);

        if(log(randnumbers::uniform()) < acc) {
          omega = omega_s;
          xi = xi_s;
          FComega.acceptance++;
        }

        // calculation of alpha
        v = 0.0;

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / omega;
          tau = sqrt((df+1)/(pow(z,2) + df));
          v += (z * tau * gsl_ran_tdist_pdf(alpha*z*tau, df + 1) / gsl_cdf_tdist_P(alpha*z*tau, df + 1)
                 - bdf * omega / pow(sqrt(1.0 + pow(alpha, 2)), 3) * (z * pow(tau,2) / omega - alpha * tau * df / (pow(z,2) + df) / omega * gsl_ran_tdist_pdf(alpha*z*tau, df + 1) / gsl_cdf_tdist_P(alpha*z*tau, df + 1)));
        }

        w = (efal[(max(-100, min(100, (int)nearbyint(alpha*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)]
            + pow(bdf * omega / pow(sqrt(1 + pow(alpha, 2)), 3), 2) *  efxi[(max(-100, min(100, (int)nearbyint(alpha*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)]);

        P = w * FCnonpp->beta.rows();

        mu = alpha + v / P;

        double alpha_s = mu + randnumbers::rand_normal() / sqrt(P);

        if (abs(alpha_s) > 20.0) {
          if (alpha_s > 0.0) {
            alpha_s = 20.0;
          }
          else {
            alpha_s = - 20.0;
          }
        }

        double d_s = alpha_s / sqrt(1 + pow(alpha_s, 2));
        xi_s = bdf * omega * d_s;

        v = 0.0;
        acc = 0.0;

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / omega;
          z_s = ((*betap) + xi_s - avg) / omega;
          tau = sqrt((df+1)/(pow(z,2) + df));
          tau_s = sqrt((df+1)/(pow(z_s,2) + df));
          v += (z_s * tau_s * gsl_ran_tdist_pdf(alpha_s*z_s*tau_s, df + 1) / gsl_cdf_tdist_P(alpha_s*z_s*tau_s, df + 1)
                  - bdf * omega / pow(sqrt(1.0 + pow(alpha_s, 2)), 3) * (z_s * pow(tau_s,2) / omega - alpha_s * tau_s * df / (pow(z_s,2) + df) / omega * gsl_ran_tdist_pdf(alpha_s*z_s*tau_s, df + 1) / gsl_cdf_tdist_P(alpha_s*z_s*tau_s, df + 1)));
          acc += - 0.5*(df+1)*log(1 + pow(z_s,2)/df) + log(gsl_cdf_tdist_P(alpha_s*z_s*tau_s, df+1)) + 0.5*(df+1)*log(1 + pow(z,2)/df) - log(gsl_cdf_tdist_P(alpha*z*tau, df+1));
        }

        w = (efal[(max(-100, min(100, (int)nearbyint(alpha_s*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)]
            + pow(bdf * omega / pow(sqrt(1 + pow(alpha_s, 2)), 3), 2) *  efxi[(max(-100, min(100, (int)nearbyint(alpha_s*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)]);

        P_s = w * FCnonpp->beta.rows();

        mu_s = alpha_s + v / P_s;

        acc += 0.5 * pow(alpha_s - mu, 2) * P - 0.5 * pow(alpha - mu_s, 2) * P_s + 0.5 * log(P_s) - 0.5 * log(P);

        acc += (- 0.5 * pow(alpha_s, 2) + 0.5 * pow(alpha, 2)) / priorskew;

        if (log(randnumbers::uniform()) < acc) {
          alpha = alpha_s;
          delta = d_s;
          xi = xi_s;
          FCalpha.acceptance++;
        }

        psi = alpha;

        // calculation of df
        v = 0.0;
        double tval = 0.0;
        double Tval = 0.0;
        double tmp = 0.0;
        double iota = 0.0;

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / omega;
          tau = sqrt((df+1)/(pow(z,2) + df));
          tmp = alpha*z*tau;
          tval = gsl_ran_tdist_pdf(tmp, df + 1.0);
          Tval = gsl_cdf_tdist_P(tmp, df+1.0);
          iota = io[(max(-100, min(100, (int)nearbyint(tmp*2.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)];

          v = v + (0.5 * (randnumbers::digamma_exact(dfhalfplusone) - randnumbers::digamma_exact(dfhalf) - (2 * df + 1) / (df + 1) / df - log(1.0 + pow(z,2) / df)
                        + pow(z * tau, 2) / df + alpha * z * (z * z - 1) / pow(df+z*z, 2) / tau * tval / Tval + iota / Tval)
                 - 0.5 * omega * delta * bdf * (randnumbers::digamma_exact(dfminusonehalf) - randnumbers::digamma_exact(dfhalf) + 1.0 / df) * (z * pow(tau,2) / omega - alpha * tau * df / (pow(z,2) + df) / omega * tval / Tval)) * df;
        }

        w = (efdf[(max(-200, min(500, (int)nearbyint(log(df)*100.0))) + 200)]
            + pow( 0.5 * omega * delta * bdf * (randnumbers::digamma_exact(dfminusonehalf) - randnumbers::digamma_exact(dfhalf) + 1.0 / df), 2) * efxi[(max(-100, min(100, (int)nearbyint(alpha*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df)*10.0))) + 20)]) * pow(df, 2);

        P = w * FCnonpp->beta.rows();

        mu = log(df) + v / P;

        double df_s = exp(mu + randnumbers::rand_normal() / sqrt(P));

        if (df_s > 100.0) {
          df_s = 100.0;
        }

        double dfsminusonehalf = 0.5 * (df_s - 1.0);
        double dfshalf = 0.5 * df_s;
        double dfsplusonehalf = 0.5 * (df_s + 1.0);
        double dfshalfplusone = 0.5 * df_s + 1.0;

        double bdf_s = sqrt(df_s / PI) * randnumbers::gamma_exact(dfsminusonehalf) / randnumbers::gamma_exact(dfshalf);

        xi_s = omega * delta * bdf_s;

        v = 0.0;
        acc = 0.0;
        double tmp_s;
        double tval_s;
        double Tval_s;

        betap = FCnonpp->beta.getV();
        for (int i = 0; i < FCnonpp->beta.rows(); i++, betap++) {
          z = ((*betap) + xi - avg) / omega;
          z_s = ((*betap) + xi_s - avg) / omega;
          tau = sqrt((df+1)/(pow(z,2) + df));
          tau_s = sqrt((df_s+1)/(pow(z_s,2) + df_s));
          tmp = alpha*z*tau;
          tmp_s = alpha*z_s*tau_s;
          tval_s = gsl_ran_tdist_pdf(tmp_s, df_s + 1.0);
          Tval_s = gsl_cdf_tdist_P(tmp_s, df_s+1.0);
          Tval = gsl_cdf_tdist_P(tmp, df+1.0);
          iota = io[(max(-100, min(100, (int)nearbyint(tmp_s*2.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df_s)*10.0))) + 20)];
          v += (0.5 * (randnumbers::digamma_exact(dfshalfplusone) - randnumbers::digamma_exact(dfshalf) - (2 * df_s + 1) / (df_s + 1) / df_s - log(1.0 + pow(z_s,2) / df_s)
                        + pow(z_s * tau_s, 2) / df_s + alpha * z_s * (z_s * z_s - 1) / pow(df_s+z_s*z_s, 2) / tau_s * tval_s / Tval_s + iota / Tval_s)
                - 0.5 * omega * delta * bdf_s * (randnumbers::digamma_exact(dfsminusonehalf) - randnumbers::digamma_exact(dfshalf) + 1.0 / df_s) * (z_s * pow(tau_s,2) / omega - alpha * tau_s * df_s / (pow(z_s,2) + df_s) / omega * tval_s / Tval_s)) * df_s;
          acc += - 0.5*log(df_s) + randnumbers::lngamma_exact(dfsplusonehalf) - randnumbers::lngamma_exact(dfshalf) - 0.5*(df_s+1)*log(1 + pow(z_s,2)/df_s) + log(Tval_s) - (- 0.5*log(df) + randnumbers::lngamma_exact(dfplusonehalf) - randnumbers::lngamma_exact(dfhalf) - 0.5*(df+1)*log(1 + pow(z,2)/df) + log(Tval));
        }

        w = (efdf[(max(-200, min(500, (int)nearbyint(log(df_s)*100.0))) + 200)]
            + pow( 0.5 * omega * delta * bdf_s * (randnumbers::digamma_exact(dfsminusonehalf) - randnumbers::digamma_exact(dfshalf) + 1.0 / df_s), 2) * efxi[(max(-100, min(100, (int)nearbyint(alpha*10.0))) + 100)][(max(-20, min(50, (int)nearbyint(log(df_s)*10.0))) + 20)]) * pow(df_s, 2);

        P_s = w * FCnonpp->beta.rows();

        mu_s = log(df_s) + v / P_s;

        acc += 0.5 * pow(log(df_s) - mu, 2) * P - 0.5 * pow(log(df) - mu_s, 2) * P_s + 0.5 * log(P_s) - 0.5 * log(P);

        acc += (- 0.5 * pow(log(df_s) - priordfmean, 2) + 0.5 * pow(log(df) - priordfmean, 2)) / priordf;

        if (log(randnumbers::uniform()) < acc && (df_s > 4.0)) {
          df = df_s;
          xi = xi_s;
          FCdf.acceptance++;
        }

        // calculation of additional values
        FComega.beta(0,0) = omega;
        FCalpha.beta(0,0) = alpha;
        FCdf.beta(0,0) = df;

        double sk = 0.0;
        double ku = 0.0;
        double sigZ = 0.0;
        dfhalf = 0.5 * df;
        dfminusonehalf = 0.5 * (df - 1.0);
        bdf = sqrt(df / PI) * randnumbers::gamma_exact(dfminusonehalf) / randnumbers::gamma_exact(dfhalf);
        sigZ = sqrt(df/(df-2)-pow(bdf*delta,2));
        if(df > 3){
          sk = (df*(3-delta*delta)/(df-3)-3.0*df/(df-2)+2*pow(bdf*delta,2))*bdf*delta/pow(sqrt(sigZ),3);
        }
        if(df > 4){
          ku = (3*pow(df,2)/(df-2)/(df-4)-4*pow(bdf*delta,2)*df*(3-pow(delta,2))/(df-3)+6*pow(bdf*delta,2)*df/(df-2)-3*pow(bdf*delta,4))/pow(sigZ,4);
        }
        FCskew.beta(0,0) = sk;
        FCkurt.beta(0,0) = ku;

        FCskew.acceptance++;
        FCkurt.acceptance++;

        FChrandom->df = df;
        FChrandom->omega = FComega.beta(0,0);
        FChrandom->alpha = FCalpha.beta(0,0);
        FChrandom->xi = xi;

        FComega.update();
        FCalpha.update();
        FCdf.update();
        FCskew.update();
        FCkurt.update();

        beta(0,0) = pow(FComega.beta(0,0),2) * (df / (df - 2.0) - pow(bdf * delta, 2));
      }
      else {
        beta(0,0) = rand_invgamma(a_invgamma+0.5*designp->rankK,
                                        b_invgamma+0.5*compute_quadform());
      }

    if (beta(0,0) > 5)
      beta(0,0) = 5;


    beta(0,1) = likep->get_scale()/beta(0,0);

    FCnonpp->tau2 = beta(0,0);

    if (simplerandom==false)
      likepRE->sigma2=beta(0,0);

    acceptance++;
    FC::update();
    }

  }


bool FC_hrandom_variance::posteriormode(void)
  {
  return  FC_nonp_variance::posteriormode();
  }


void FC_hrandom_variance::outresults(ofstream & out_stata, ofstream & out_R, ofstream & out_R2BayesX,
                         const ST::string & pathresults)
  {
  if (lambdaconst==false) {
    // added
    int beginning = 0;
        for (int i = 0; i < pathresults.length() - 30; i++) {
          if ("variance_of_random_effect_of_" == pathresults.substr(i, 29)) {
            beginning = i;
            i = pathresults.length();
          }
        }
          if (redist == "sn") {
              const ST::string & pathresults_omega = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_omega.res";
              const ST::string & pathresults_alpha = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_alpha.res";
              const ST::string & pathresults_skew = pathresults.substr(0,beginning) + "skewness_of_random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + ".res";

              FComega.outresults(out_stata,out_R,out_R2BayesX,pathresults_omega);
              FCalpha.outresults(out_stata,out_R,out_R2BayesX,pathresults_alpha);
              FCskew.outresults(out_stata,out_R,out_R2BayesX,pathresults_skew);
          }
          if (redist == "snmh") {
              const ST::string & pathresults_omega = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_omega.res";
              const ST::string & pathresults_alpha = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_alpha.res";
              const ST::string & pathresults_skew = pathresults.substr(0,beginning) + "skewness_of_random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + ".res";

              FComega.outresults(out_stata,out_R,out_R2BayesX,pathresults_omega);
              FCalpha.outresults(out_stata,out_R,out_R2BayesX,pathresults_alpha);
              FCskew.outresults(out_stata,out_R,out_R2BayesX,pathresults_skew);
          }
          if (redist == "t") {
              const ST::string & pathresults_omega = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_omega.res";
              const ST::string & pathresults_df = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_df.res";
              const ST::string & pathresults_kurt = pathresults.substr(0,beginning) + "kurtosis_of_random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + ".res";

              FComega.outresults(out_stata,out_R,out_R2BayesX,pathresults_omega);
              FCdf.outresults(out_stata,out_R,out_R2BayesX,pathresults_df);
              FCkurt.outresults(out_stata,out_R,out_R2BayesX,pathresults_kurt);
          }
          if (redist == "tmh") {
              const ST::string & pathresults_omega = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_omega.res";
              const ST::string & pathresults_df = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_df.res";
              const ST::string & pathresults_kurt = pathresults.substr(0,beginning) + "kurtosis_of_random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + ".res";

              FComega.outresults(out_stata,out_R,out_R2BayesX,pathresults_omega);
              FCdf.outresults(out_stata,out_R,out_R2BayesX,pathresults_df);
              FCkurt.outresults(out_stata,out_R,out_R2BayesX,pathresults_kurt);
          }
          if (redist == "st") {
              const ST::string & pathresults_omega = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_omega.res";
              const ST::string & pathresults_alpha = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_alpha.res";
              const ST::string & pathresults_df = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_df.res";
              const ST::string & pathresults_skew = pathresults.substr(0,beginning) + "skewness_of_random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + ".res";
              const ST::string & pathresults_kurt = pathresults.substr(0,beginning) + "kurtosis_of_random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + ".res";

              FComega.outresults(out_stata,out_R,out_R2BayesX,pathresults_omega);
              FCalpha.outresults(out_stata,out_R,out_R2BayesX,pathresults_alpha);
              FCdf.outresults(out_stata,out_R,out_R2BayesX,pathresults_df);
              FCskew.outresults(out_stata,out_R,out_R2BayesX,pathresults_skew);
              FCkurt.outresults(out_stata,out_R,out_R2BayesX,pathresults_kurt);
          }
          if (redist == "stmh") {
              const ST::string & pathresults_omega = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_omega.res";
              const ST::string & pathresults_alpha = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_alpha.res";
              const ST::string & pathresults_df = pathresults.substr(0,beginning) + "random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + "_df.res";
              const ST::string & pathresults_skew = pathresults.substr(0,beginning) + "skewness_of_random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + ".res";
              const ST::string & pathresults_kurt = pathresults.substr(0,beginning) + "kurtosis_of_random_effect_of_" + pathresults.substr(beginning + 29, pathresults.length() - beginning - 33) + ".res";

              FComega.outresults(out_stata,out_R,out_R2BayesX,pathresults_omega);
              FCalpha.outresults(out_stata,out_R,out_R2BayesX,pathresults_alpha);
              FCdf.outresults(out_stata,out_R,out_R2BayesX,pathresults_df);
              FCskew.outresults(out_stata,out_R,out_R2BayesX,pathresults_skew);
              FCkurt.outresults(out_stata,out_R,out_R2BayesX,pathresults_kurt);
          }

    FC_nonp_variance::outresults(out_stata,out_R,out_R2BayesX,pathresults);
  }
  }

  // added
  void FC_hrandom_variance::get_samples(const ST::string & filename,ofstream & outg) const
    {
      int beginning = 0;
      for (int i = 0; i < filename.length() - 40; i++) {
        if ("variance_of_random_effect_of_" == filename.substr(i, 29)) {
          beginning = i;
          i = filename.length();
        }
      }

      if(redist == "sn"){
          const ST::string & filename_omega = filename.substr(0,beginning) + "random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_omega_sample.raw";
          const ST::string & filename_alpha = filename.substr(0,beginning) + "random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_alpha_sample.raw";
          const ST::string & filename_skew = filename.substr(0,beginning) + "skewness_of_random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_sample.raw";

          FComega.get_samples(filename_omega,outg);
          FCalpha.get_samples(filename_alpha,outg);
          FCskew.get_samples(filename_skew,outg);
      }

      if(redist == "snmh"){
          const ST::string & filename_skew = filename.substr(0,beginning) + "skewness_of_random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_sample.raw";
          FCskew.get_samples(filename_skew,outg);
      }

      if(redist == "t" || redist == "tmh"){
          const ST::string & filename_omega = filename.substr(0,beginning) + "random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_omega_sample.raw";
          const ST::string & filename_df = filename.substr(0,beginning) + "random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_df_sample.raw";
          const ST::string & filename_kurt = filename.substr(0,beginning) + "kurtosis_of_random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_df_sample.raw";

          FComega.get_samples(filename_omega,outg);
          FCdf.get_samples(filename_df,outg);
          FCkurt.get_samples(filename_kurt,outg);
      }

      if(redist == "st" || redist == "tmh"){
          const ST::string & filename_omega = filename.substr(0,beginning) + "random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_omega_sample.raw";
          const ST::string & filename_alpha = filename.substr(0,beginning) + "random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_alpha_sample.raw";
          const ST::string & filename_skew = filename.substr(0,beginning) + "skewness_of_random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_sample.raw";
          const ST::string & filename_df = filename.substr(0,beginning) + "random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_df_sample.raw";
          const ST::string & filename_kurt = filename.substr(0,beginning) + "kurtosis_of_random_effect_of_" + filename.substr(beginning + 29, filename.length() - beginning - 40) + "_df_sample.raw";

          FComega.get_samples(filename_omega,outg);
          FCalpha.get_samples(filename_alpha,outg);
          FCskew.get_samples(filename_skew,outg);
          FCdf.get_samples(filename_df,outg);
          FCkurt.get_samples(filename_kurt,outg);
      }


    FC::get_samples(filename,outg);
    }

//------------------------------------------------------------------------------
//---------------------- CLASS: FC_hrandom_variance_ssvs -----------------------
//------------------------------------------------------------------------------

void FC_hrandom_variance_ssvs::read_options(vector<ST::string> & op,
vector<ST::string> & vn)
  {

  int f;

  /*

  39 abeta
  40 bbeta
  41 r
  42 v
  43 aQ
  44 bQ

  */

  FC_nonp_variance::read_options(op,vn);
  FC_hrandom_variance::read_options(op,vn);

  f = op[39].strtodouble(abeta);
  f = op[40].strtodouble(bbeta);
  f = op[41].strtodouble(r);
  f = op[45].strtolong(regiterates);
  }


FC_hrandom_variance_ssvs::FC_hrandom_variance_ssvs(void)
  {

  }



FC_hrandom_variance_ssvs::FC_hrandom_variance_ssvs(MASTER_OBJ * mp,unsigned & enr,
                  GENERAL_OPTIONS * o,DISTR * lp,
                  DISTR * lpRE,
                  const ST::string & t,const ST::string & fp,
                  DESIGN * Dp,FC_nonp * FCn,vector<ST::string> & op,
                  vector<ST::string> & vn)
     : FC_hrandom_variance(mp,enr, o,lp,lpRE,t,fp,Dp,FCn,op,vn)
  {
  read_options(op,vn);

  FC_delta = FC(o,"",FCn->beta.rows(),1,"");
  FC_delta.setbeta(FCn->beta.rows(),1,1);

  FC_omega = FC(o,"",1,1,"");
  FC_omega.setbeta(1,1,0.5);

  pen = datamatrix(FCn->beta.rows(),1,1);
  }


FC_hrandom_variance_ssvs::FC_hrandom_variance_ssvs(
   const FC_hrandom_variance_ssvs & m)
  : FC_hrandom_variance(FC_hrandom_variance(m))
  {
  FC_delta = m.FC_delta;
  FC_omega = m.FC_omega;
  abeta = m.abeta;
  bbeta = m.bbeta;
  r = m.r;
  regiterates = m.regiterates;
  pen = m.pen;
  }


const FC_hrandom_variance_ssvs & FC_hrandom_variance_ssvs::operator=(
const FC_hrandom_variance_ssvs & m)
  {

  if (this==&m)
	 return *this;
  FC_hrandom_variance::operator=(FC_hrandom_variance(m));
  FC_delta = m.FC_delta;
  FC_omega = m.FC_omega;
  abeta = m.abeta;
  bbeta = m.bbeta;
  r = m.r;
  regiterates = m.regiterates;
  pen = m.pen;
  return *this;
  }


void FC_hrandom_variance_ssvs::update(void)
  {

  register unsigned i;

  double * workbetafcn = FCnonpp->beta.getV();
  double * workdelta = FC_delta.beta.getV();

  double * linpredREp;
  if (likepRE->linpred_current==1)
    linpredREp = likepRE->linearpred1.getV();
  else
    linpredREp = likepRE->linearpred2.getV();

  double * ww = likepRE->workingweight.getV();

  double * penp = pen.getV();

  double L;
  double sumdelta=0;
  double diff2;
  double quadform = 0;
  double rsqrt = sqrt(r);
  double rinv = 1/r;
  double pr;

  double omega = FC_omega.beta(0,0);

  int nrcluster = FCnonpp->beta.rows();

  // update deltas
  for (i=0;i<nrcluster;i++,penp++,workbetafcn++,ww++,workdelta++,linpredREp++)
    {

    diff2 = pow((*workbetafcn) - (*linpredREp),2);

    L = 1/rsqrt*exp(1/(2*(beta(0,0)))*diff2*(1-1/r)) ;
    pr = 1/(1+ (1-omega)/omega*L);

    *workdelta = randnumbers::bernoulli(pr);

    sumdelta+=(*workdelta);

    if (*workdelta == 0)
      {
      *ww = rinv;
      *penp = r;
      quadform += rinv*diff2;
      }
    else
      {
      *ww = 1;
      *penp = 1;
       quadform += diff2;
      }
    }

  b_invgamma = masterp->level1_likep[equationnr]->trmult*b_invgamma_orig;

  beta(0,0) = rand_invgamma(a_invgamma+0.5*nrcluster,b_invgamma+0.5*quadform);

  beta(0,1) = likep->get_scale()/beta(0,0);

  // update omega
  FC_omega.beta(0,0) = randnumbers::rand_beta(abeta+sumdelta,bbeta+nrcluster-sumdelta);

  FCnonpp->tau2 = beta(0,0);

  designp->compute_penalty2(pen);

  likepRE->sigma2 = beta(0,0);

  acceptance++;

  FC_delta.update();
  FC_omega.update();
  FC::update();

  }


void FC_hrandom_variance_ssvs::outoptions(void)
  {

  optionsp->out("  Options for spike and slap prior\n");
  optionsp->out("\n");

  FC_nonp_variance::outoptions();

  optionsp->out("  Hyperparameter a for beta distribution: " +
                ST::doubletostring(abeta) + "\n" );
  optionsp->out("  Hyperparameter b for beta distribution: " +
                ST::doubletostring(bbeta) + "\n" );

  optionsp->out("  Spike and slap parameter r: " + ST::doubletostring(r) + "\n");

  optionsp->out("\n");

  }


void FC_hrandom_variance_ssvs::outresults(ofstream & out_stata,
                                              ofstream & out_R, ofstream & out_R2BayesX,
                                              const ST::string & pathresults)
  {

  if (pathresults.isvalidfile() != 1)
    {

    ST::string pathresults_delta = pathresults.substr(0,pathresults.length()-4) + "_delta.res";
    ST::string pathresults_omega = pathresults.substr(0,pathresults.length()-4) + "_omega.res";

    FC_hrandom_variance::outresults(out_stata,out_R,out_R2BayesX,pathresults);

    FC_delta.outresults(out_stata,out_R,out_R2BayesX,"");
    FC_omega.outresults(out_stata,out_R,out_R2BayesX,pathresults_omega);

    optionsp->out("    Results for the inclusion probabilities are stored in file\n");
    optionsp->out("    " +  pathresults_delta + "\n");
    optionsp->out("\n");
    optionsp->out("\n");

    optionsp->out("    Inclusion probability parameter omega:\n");
    optionsp->out("\n");
    FC_omega.outresults_singleparam(out_stata,out_R,"");
    optionsp->out("    Results for the inclusion probability parameter omega are also stored in file\n");
    optionsp->out("    " +  pathresults_omega + "\n");
    optionsp->out("\n");
    optionsp->out("\n");

    // deltas
    ofstream ou(pathresults_delta.strtochar());

    ou << "intnr   " << designp->datanames[0]  << "   pmean" << endl;

    unsigned i;
    double * deltameanp = FC_delta.betamean.getV();

    for (i=0;i<FC_delta.beta.rows();i++,deltameanp++)
      ou << (i+1) << "  " << designp->effectvalues[i]  << "  " << (*deltameanp) << endl;
    }

  }


void FC_hrandom_variance_ssvs::get_samples(
   const ST::string & filename,ofstream & outg) const
  {
  FC_hrandom_variance::get_samples(filename,outg);

  ST::string filename_delta = filename.substr(0,filename.length()-4) + "_delta.raw";
  FC_delta.get_samples(filename_delta,outg);

  ST::string filename_omega = filename.substr(0,filename.length()-4) + "_omega.raw";
  FC_omega.get_samples(filename_omega,outg);
  }


void FC_hrandom_variance_ssvs::compute_autocorr_all(const ST::string & path,
                                           unsigned lag, ofstream & outg) const
  {
  FC_hrandom_variance::compute_autocorr_all(path,lag,outg);

//  ST::string path_delta = path.substr(0,path.length()-4) + "_delta.raw";
//  FC_delta.compute_autocorr_all(path_delta,lag,outg);

  ST::string path_omega = path.substr(0,path.length()-4) + "_omega.raw";
  FC_omega.compute_autocorr_all(path_omega,lag,outg);
  }



} // end: namespace MCMC
