
    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_df --------------------------------------
    //------------------------------------------------------------------------------

    // Constructor:
    DISTR_bivst_df::DISTR_bivst_df(GENERAL_OPTIONS * o,
                                         const datamatrix & r,
                                         unsigned & p,
                                         const datamatrix & w)
    : DISTR_gamlss(o,r,7,w) // <-----
    {
        family = "Bivariate Skew-t Distribution - DF";
        outpredictor = true;
        outexpectation = false; // only true if main parameter
        predictor_name = "df";

        STOREDVALUES:get_bivst_ef_df_values(ef);

        linpredminlimit=-10; // necessary for log-link
        linpredmaxlimit=15; // necessary for log-link

        // updateIWLS = false; // only false and necessary if gibbs sampling?
    }


    // Copy Constructor:
    DISTR_bivst_df::DISTR_bivst_df(const DISTR_bivst_alpha & nd)
    : DISTR_gamlss(DISTR_gamlss(nd))
    {
        response2 = nd.response2;
        response2p = nd.response2p;
    }

    // Operator:
    const DISTR_bivst_df & DISTR_bivst_df::operator=(
                                                           const DISTR_bivst_df & nd)
    {
        if (this==&nd)
            return *this;
        DISTR_gamlss::operator=(DISTR_gamlss(nd));
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_df_values(ef);
        return *this;
    }


    double DISTR_bivst_df::get_intercept_start(void)
    {
        return 0;
    }

    void DISTR_bivst_df::compute_param_mult(vector<double *>  linpred,double * param)
    {
        *param = exp(*linpred[0]); // <--- c.f. linpred info
    }


    void DISTR_bivst_df::set_worklin(void)
    {

        DISTR_gamlss::set_worklin();

        response2p = response2.getV();

    }



    void DISTR_bivst_df::modify_worklin(void)
    {

        DISTR_gamlss::modify_worklin();

        if (counter<nrobs-1)
        {
            response2p++;
        }

    }


    double DISTR_bivst_df::loglikelihood_weightsone(double * response,
                                                       double * linpred)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double df = exp(*linpred);
        double al2 = (*worklin[0]);
        double al1 = (*worklin[1]);
        double r = (*worklin[2]) / sqrt(1 + pow((*worklin[2]), 2));
        double r2 = r * r;
        double om2 = exp((*worklin[3]));
        double om1 = exp((*worklin[4]));
        double xi2 = (*worklin[5]);
        double xi1 = (*worklin[6]);

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;
        double dfhalf = df * 0.5;

        double l = 0;

        l += - log(df);
        l += randnumbers::lngamma_exact(dfplustwohalf) - randnumbers::lngamma_exact(dfhalf) - dfplustwohalf * log(1.0 + Q / df);
        l += log(gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0)); // ll, simplified

        modify_worklin();

        return l;

    }


    void DISTR_bivst_df::compute_iwls_wweightschange_weightsone(
                                                                   double * response,
                                                                   double * linpred,
                                                                   double * workingweight,
                                                                   double * workingresponse,
                                                                   double & like,
                                                                   const bool & compute_like)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double df = exp(*linpred);
        double al2 = (*worklin[0]);
        double al1 = (*worklin[1]);
        double r = (*worklin[2]) / sqrt(1 + pow((*worklin[2]), 2));
        double om2 = exp((*worklin[3]));
        double om1 = exp((*worklin[4]));
        double xi2 = (*worklin[5]);
        double xi1 = (*worklin[6]);

        double r2 = r * r;

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;
        double dfhalf = df * 0.5;

        double tval = ;
        double Tval = gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);

        double nu = ; // v

        if(nu != nu){
            nu = 0.0;
        } // prevent nans (might only happen if *workingweight is fixed to a small value

        *workingweight = ; // w

        *workingresponse = *linpred + nu/(*workingweight);

        if (compute_like)
        {

            like += - log(df) + randnumbers::lngamma_exact(dfplustwohalf) - randnumbers::lngamma_exact(dfhalf) - dfplustwohalf * log(1.0 + Q / df) + log(Tval); // loglikelihood

        }

        modify_worklin();

    }

    void DISTR_bivst_df::outoptions(void)
    {
        DISTR::outoptions();
        optionsp->out("  Link function (df): log\n");
        optionsp->out("\n");
        optionsp->out("\n");
    }



    void DISTR_bivst_df::update_end(void)
    {


        // helpmat1 stores (eta_mu)

        double * worklin;
        if (linpred_current==1)
            worklin = linearpred1.getV();
        else
            worklin = linearpred2.getV();

        double * pmu = helpmat1.getV();

        unsigned i;
        for (i=0;i<nrobs;i++,pmu++,worklin++)
        {
            *pmu = exp(*worklin);
            //    double t = 0;
        }

    }



    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_alpha --------------------------------------
    //------------------------------------------------------------------------------

    // Constructor:
    DISTR_bivst_alpha::DISTR_bivst_alpha(GENERAL_OPTIONS * o,
                                         const datamatrix & r,
                                         unsigned & p,
                                         const datamatrix & w)
    : DISTR_gamlss(o,r,7,w) // <-----
    {
        pos = p;
        family = "Bivariate Skew-t Distribution - Alpha";
        outpredictor = true;
        outexpectation = false; // only true if main parameter
        predictor_name = "alpha";

        STOREDVALUES:get_bivst_ef_alpha_values(ef);

        linpredminlimit=-25; // necessary for log-link
        linpredmaxlimit=25; // necessary for log-link

        // updateIWLS = false; // only false and necessary if gibbs sampling?
    }


    // Copy Constructor:
    DISTR_bivst_alpha::DISTR_bivst_alpha(const DISTR_bivst_alpha & nd)
    : DISTR_gamlss(DISTR_gamlss(nd))
    {
        pos = nd.pos;
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_alpha_values(ef);
    }

    // Operator:
    const DISTR_bivst_alpha & DISTR_bivst_alpha::operator=(
                                                           const DISTR_bivst_alpha & nd)
    {
        if (this==&nd)
            return *this;
        DISTR_gamlss::operator=(DISTR_gamlss(nd));
        pos = nd.pos;
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_alpha_values(ef);
        return *this;
    }


    double DISTR_bivst_alpha::get_intercept_start(void)
    {
        return 0;
    }

    void DISTR_bivst_alpha::compute_param_mult(vector<double *>  linpred,double * param)
    {
        *param = (*linpred[1 + pos]); // <--- c.f. linpred info
    }


    void DISTR_bivst_alpha::set_worklin(void)
    {

        DISTR_gamlss::set_worklin();

        response2p = response2.getV();

    }



    void DISTR_bivst_alpha::modify_worklin(void)
    {

        DISTR_gamlss::modify_worklin();

        if (counter<nrobs-1)
        {
            response2p++;
        }

    }


    double DISTR_bivst_alpha::loglikelihood_weightsone(double * response,
                                                       double * linpred)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double df = exp(*worklin[0]);
        double al2 = (*worklin[1]);
        double al1 = (*linpred);
        double r = (*worklin[2]) / sqrt(1 + pow((*worklin[2]), 2));
        double r2 = r * r;
        double om2 = exp((*worklin[3]));
        double om1 = exp((*worklin[4]));
        double xi2 = (*worklin[5]);
        double xi1 = (*worklin[6]);

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;

        double l = 0.0;

        l += log(gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0));
        // ll, simplified

        modify_worklin();

        return l;

    }


    void DISTR_bivst_alpha::compute_iwls_wweightschange_weightsone(
                                                                   double * response,
                                                                   double * linpred,
                                                                   double * workingweight,
                                                                   double * workingresponse,
                                                                   double & like,
                                                                   const bool & compute_like)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double df = exp(*worklin[0]);
        double al2 = (*worklin[1]);
        double al1 = (*linpred);
        double r = (*worklin[2]) / sqrt(1 + pow((*worklin[2]), 2));
        double om2 = exp((*worklin[3]));
        double om1 = exp((*worklin[4]));
        double xi2 = (*worklin[5]);
        double xi1 = (*worklin[6]);

        double r2 = r * r;

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;

        double tval = gsl_ran_tdist_pdf((alpha1 * z1 + alpha2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);
        double Tval = gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);

        double nu = tval / Tval * z1 * sqrt((df + 2.0) / (df + Q)); // v

        if(nu != nu){
            nu = 0.0;
        } // prevent nans (might only happen if *workingweight is fixed to a small value

        *workingweight = ef[max(-9, min(9, (int)nearbyint(r * 10))) + 9][max(-5, min(5, (int)nearbyint(al1*0.5))) + 5][max(-5, min(5, (int)nearbyint(al2*0.5))) + 5][max(-10, min(24, (int)nearbyint(log(df) * 5))) + 10]; // w

        *workingresponse = *linpred + nu/(*workingweight);

        if (compute_like)
        {

            like +=  log(Tval); // loglikelihood

        }

        modify_worklin();

    }

    void DISTR_bivst_alpha::outoptions(void)
    {
        DISTR::outoptions();
        optionsp->out("  Link function (alpha): identity\n");
        optionsp->out("\n");
        optionsp->out("\n");
    }



    void DISTR_bivst_alpha::update_end(void)
    {


        // helpmat1 stores (eta_mu)

        double * worklin;
        if (linpred_current==1)
            worklin = linearpred1.getV();
        else
            worklin = linearpred2.getV();

        double * pmu = helpmat1.getV();

        unsigned i;
        for (i=0;i<nrobs;i++,pmu++,worklin++)
        {
            *pmu = (*worklin);
            //    double t = 0;
        }

    }




    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_rho ----------------------------------------
    //------------------------------------------------------------------------------

    // Constructor:
    DISTR_bivst_rho::DISTR_bivst_rho(GENERAL_OPTIONS * o,
                                     const datamatrix & r,
                                     const datamatrix & w)
    : DISTR_gamlss(o,r,7,w) // <-----
    {
        family = "Bivariate Skew-t Distribution - Rho";
        outpredictor = true;
        outexpectation = false; // only true if main parameter
        predictor_name = "rho";

        STOREDVALUES:get_bivst_ef_rho_values(ef);

        linpredminlimit=-100; // necessary for log-link
        linpredmaxlimit=100; // necessary for log-link

        // updateIWLS = false; // only false and necessary if gibbs sampling?
    }


    // Copy Constructor:
    DISTR_bivst_rho::DISTR_bivst_rho(const DISTR_bivst_rho & nd)
    : DISTR_gamlss(DISTR_gamlss(nd))
    {
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_rho_values(ef);
    }

    // Operator:
    const DISTR_bivst_rho & DISTR_bivst_rho::operator=(
                                                       const DISTR_bivst_rho & nd)
    {
        if (this==&nd)
            return *this;
        DISTR_gamlss::operator=(DISTR_gamlss(nd));
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_rho_values(ef);
        return *this;
    }

    double DISTR_bivst_rho::get_intercept_start(void)
    {
        return 0;
    }

    void DISTR_bivst_rho::compute_param_mult(vector<double *>  linpred,double * param)
    {
        double arg = (*linpred[3]);
        *param = arg/pow(1+pow(arg,2),0.5); // <--- c.f. linpred info
    }


    void DISTR_bivst_rho::set_worklin(void)
    {

        DISTR_gamlss::set_worklin();

        response2p = response2.getV();

    }



    void DISTR_bivst_rho::modify_worklin(void)
    {

        DISTR_gamlss::modify_worklin();

        if (counter<nrobs-1)
        {
            response2p++;
        }

    }

    double DISTR_bivst_rho::loglikelihood_weightsone(double * response,
                                                     double * linpred)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double r = 0;

        if (*linpred <= -100)
            r  = -0.99995;
        else if (*linpred >= 100)
            r  = 0.99995;
        else
            r = (*linpred)/pow((1+pow((*linpred),2)),0.5);

        double r2 = r * r;
        double df = exp(*worklin[0]);
        double al2 = (*worklin[1]);
        double al1 = (*worklin[2]);
        double om2 = exp((*worklin[3]));
        double om1 = exp((*worklin[4]));
        double xi2 = (*worklin[5]);
        double xi1 = (*worklin[6]);

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;

        double l = 0.0;

        l += - 0.5 * log(oneminrhotwo);
        l += - dfplustwohalf * log(1.0 + Q / df);
        l += log(gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0));
        // calculate log-likelihood (possibly simplified)

        modify_worklin();

        return l;

    }


    void DISTR_bivst_rho::compute_iwls_wweightschange_weightsone(
                                                                 double * response,
                                                                 double * linpred,
                                                                 double * workingweight,
                                                                 double * workingresponse,
                                                                 double & like,
                                                                 const bool & compute_like)
    {

        if (counter==0)
        {
            set_worklin();
        }

        double r = 0;

        if (*linpred <= -100)
            r  = -0.99995;
        else if (*linpred >= 100)
            r  = 0.99995;
        else
            r = (*linpred)/pow((1+pow((*linpred),2)),0.5);

        double r2 = r * r;
        double df = exp(*worklin[0]);
        double al2 = (*worklin[1]);
        double al1 = (*worklin[2]);
        double om2 = exp((*worklin[3]));
        double om1 = exp((*worklin[4]));
        double xi2 = (*worklin[5]);
        double xi1 = (*worklin[6]);

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;

        double dlinkf = 1.0 / sqrt(1.0 + pow(max(-100.0, min(100.0, (*linpred))), 2)) * (1 - r2);

        double tval = gsl_ran_tdist_pdf((alpha1 * z1 + alpha2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);
        double Tval = gsl_cdf_tdist_P((alpha1 * z1 + alpha2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);

        double nu = (r / oneminrhotwo + (- (df + 2.0) / (df + Q) + tval / Tval * (al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)) / (df + Q)) * (r / oneminrhotwo / oneminrhotwo * (z1 * z1 - 2.0 * r * z1 * z2 + z2 * z2) - 1 / oneminrhotwo * z1 * z2)) * dlinkf; // v

        if(nu != nu){
            nu = 0;} // prevent nans (might only happen if *workingweight is fixed to a small value

        *workingweight = ef[max(-9, min(9, (int)nearbyint(r * 10))) + 9][max(-5, min(5, (int)nearbyint(al1*0.5))) + 5][max(-5, min(5, (int)nearbyint(al2*0.5))) + 5][max(-10, min(24, (int)nearbyint(log(df) * 5))) + 10]; // w

        *workingresponse = *linpred + nu/(*workingweight);

        if (compute_like)
        {

            like += - 0.5 * log(oneminrhotwo) - dfplustwohalf * log(1.0 + Q / df) + log(Tval);

        }


        modify_worklin();

    }

    void DISTR_bivst_rho::outoptions(void)
    {
        DISTR::outoptions();
        optionsp->out("  Response function (rho): rhogit\n");
        optionsp->out("\n");
        optionsp->out("\n");
    }


    void DISTR_bivst_rho::update_end(void)
    {


        // helpmat1 stores (eta_mu)

        double * worklin;
        if (linpred_current==1)
            worklin = linearpred1.getV();
        else
            worklin = linearpred2.getV();

        double * pmu = helpmat1.getV();

        unsigned i;
        for (i=0;i<nrobs;i++,pmu++,worklin++)
        {
            *pmu = (*worklin)/pow((1+pow((*worklin),2)),0.5);
            //    double t = 0;
        }

    }


    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_omega --------------------------------------
    //------------------------------------------------------------------------------

    // Constructor:
    DISTR_bivst_omega::DISTR_bivst_omega(GENERAL_OPTIONS * o,
                                         const datamatrix & r,
                                         unsigned &p,
                                         const datamatrix & w)
    : DISTR_gamlss(o,r,7,w) // <-----
    {
        pos = p;
        family = "Bivariate Skew-t Distribution - Omega";
        outpredictor = true;
        outexpectation = false; // only true if main parameter
        predictor_name = "omega";

        STOREDVALUES:get_bivst_ef_omega_values(ef);

        linpredminlimit=-10; // necessary for log-link
        linpredmaxlimit=15; // necessary for log-link

        // updateIWLS = false; // only false and necessary if gibbs sampling?
    }


    // Copy Constructor:
    DISTR_bivst_omega::DISTR_bivst_omega(const DISTR_bivst_omega & nd)
    : DISTR_gamlss(DISTR_gamlss(nd))
    {
        pos = nd.pos;
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_omega_values(ef);
    }

    // Operator:
    const DISTR_bivst_omega & DISTR_bivst_omega::operator=(
                                                           const DISTR_bivst_omega & nd)
    {
        if (this==&nd)
            return *this;
        DISTR_gamlss::operator=(DISTR_gamlss(nd));
        pos = nd.pos;
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_omega_values(ef);
        return *this;
    }

    double DISTR_bivst_omega::get_intercept_start(void)
    {
        return 0;
    }

    void DISTR_bivst_omega::compute_param_mult(vector<double *>  linpred,double * param)
    {
        *param = exp(*linpred[4 + pos]); // <--- c.f. linpred info
    }

    void DISTR_bivst_omega::set_worklin(void)
    {

        DISTR_gamlss::set_worklin();

        response2p = response2.getV();

    }



    void DISTR_bivst_omega::modify_worklin(void)
    {

        DISTR_gamlss::modify_worklin();

        if (counter<nrobs-1)
        {
            response2p++;
        }

    }

    double DISTR_bivst_omega::loglikelihood_weightsone(double * response,
                                                       double * linpred)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double df = exp(*worklin[0]);
        double al2 = (*worklin[1]);
        double al1 = (*worklin[2]);
        double r = (*worklin[3]) / sqrt(1 + pow((*worklin[3]), 2));
        double r2 = r * r;
        double om2 = exp((*worklin[4]));
        double om1 = exp((*linpred));
        double xi2 = (*worklin[5]);
        double xi1 = (*worklin[6]);

        double l;

        l += - log(om1);
        l += - dfplustwohalf * log(1.0 + Q / df);
        l += log(gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0));
        // calculate log-likelihood (possibly simplified)

        modify_worklin();

        return l;

    }


    void DISTR_bivst_omega::compute_iwls_wweightschange_weightsone(
                                                                   double * response,
                                                                   double * linpred,
                                                                   double * workingweight,
                                                                   double * workingresponse,
                                                                   double & like,
                                                                   const bool & compute_like)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double df = exp(*worklin[0]);
        double al2 = (*worklin[1]);
        double al1 = (*worklin[2]);
        double r = (*worklin[3]) / sqrt(1 + pow((*worklin[3]), 2));
        double r2 = r * r;
        double om2 = exp((*worklin[4]));
        double om1 = exp((*linpred));
        double xi2 = (*worklin[5]);
        double xi1 = (*worklin[6]);

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;

        double tval = gsl_ran_tdist_pdf((alpha1 * z1 + alpha2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);
        double Tval = gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);

        double nu = (- 1.0 - om1 * z1 * (- (df + 2.0) / 2.0 * (2.0 * z1 - 2 * r * z2) / (1.0 + Q / df) / df / oneminrhotwo
                     + tval / Tval * (al1 * sqrt((df + 2.0) / (df + Q)) - (al1 * z1 + al2 * z2) * sqrt(df + 2.0) * (z1 - r * z2) / sqrt(df + Q) / (df + Q)))); // v

        if(nu != nu){
            nu = 0;
        } // prevent nans (might only happen if *workingweight is fixed to a small value

        *workingweight = ef[max(-9, min(9, (int)nearbyint(r * 10))) + 9][max(-5, min(5, (int)nearbyint(al1*0.5))) + 5][max(-5, min(5, (int)nearbyint(al2*0.5))) + 5][max(-10, min(24, (int)nearbyint(log(df) * 5))) + 10]; // w

        *workingresponse = *linpred + nu/(*workingweight);

        if (compute_like)
        {

            like += - log(om1) - dfplustwohalf * log(1.0 + Q / df) + log(Tval); // loglikelihood

        }


        modify_worklin();

    }

    void DISTR_bivst_omega::outoptions(void)
    {
        DISTR::outoptions();
        optionsp->out("  Link function (omega): log\n");
        optionsp->out("\n");
        optionsp->out("\n");
    }


    void DISTR_bivst_omega::update_end(void)
    {


        // helpmat1 stores (eta_mu)

        double * worklin;
        if (linpred_current==1)
            worklin = linearpred1.getV();
        else
            worklin = linearpred2.getV();

        double * pmu = helpmat1.getV();

        unsigned i;
        for (i=0;i<nrobs;i++,pmu++,worklin++)
        {
            *pmu = exp(*worklin);
            //    double t = 0;
        }

    }


    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_xi -----------------------------------------
    //------------------------------------------------------------------------------

    // ONLY FOR THE MAIN PARAMETER:
    void DISTR_bivst_xi::check_errors(void)
    {

        if (errors==false)
        {
            unsigned i=0;
            double * workresp = response.getV();
            double * workweight = weight.getV();
            while ( (i<nrobs) && (errors==false) )
            {

                if (*workweight > 0)
                {


                }
                else if (*workweight == 0)
                {
                }
                else
                {
                    errors=true;
                    errormessages.push_back("ERROR: negative weights encountered\n");
                }

                i++;
                workresp++;
                workweight++;

            }

        }

    }

    // Constructor:
    DISTR_bivst_xi::DISTR_bivst_xi(GENERAL_OPTIONS * o,
                                   const datamatrix & r,
                                   unsigned &p,
                                   const datamatrix & w)
    : DISTR_gamlss(o,r,7,w) // <-----
    {
        pos = p;
        family = "Bivariate Skew-t Distribution - Xi";
        outpredictor = true;
        outexpectation = true; // only true if main parameter
        predictor_name = "xi";

        STOREDVALUES:get_bivst_ef_xi_values(ef);

        // linpredminlimit=-10; // necessary for log-link
        // linpredmaxlimit=15; // necessary for log-link

        // updateIWLS = false; // only false and necessary if gibbs sampling?
    }


    // Copy Constructor:
    DISTR_bivst_xi::DISTR_bivst_xi(const DISTR_bivst_xi & nd)
    : DISTR_gamlss(DISTR_gamlss(nd))
    {
        pos = nd.pos;
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_xi_values(ef);
    }

    // Operator:
    const DISTR_bivst_xi & DISTR_bivst_xi::operator=(
                                                     const DISTR_bivst_xi & nd)
    {
        if (this==&nd)
            return *this;
        DISTR_gamlss::operator=(DISTR_gamlss(nd));
        pos = nd.pos;
        response2 = nd.response2;
        response2p = nd.response2p;
        STOREDVALUES:get_bivst_ef_xi_values(ef);
        return *this;
    }


    // ONLY FOR THE MAIN PARAMETER:
    void DISTR_bivst_xi::compute_deviance_mult(vector<double *> response,
                                               vector<double *> weight,
                                               vector<double *> linpred,
                                               double * deviance,
                                               vector<datamatrix*> aux)
    {
        if (*weight[7] == 0) // <---
            *deviance=0;
        else
        {
            double df = exp(*linpred[0]);
            double al2 = (*linpred[1]);
            double al1 = (*linpred[2]);
            double r = (*linpred[3]) / sqrt(1 + pow((*linpred[3]), 2));
            double r2 = r * r;
            double om2 = exp((*linpred[4]));
            double om1 = exp((*linpred[5]));
            double xi2 = (*linpred[6]);
            double xi1 = (*linpred[7]);

            double oneminrhotwo = 1.0 - r2;
            double z1 = (*response[0] - xi1) / om1;
            double z2 = (*response[1] - xi2) / om2;
            double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
            double dfplustwohalf = (df + 2.0) * 0.5;
            double dfhalf = df * 0.5;

            double l = 0;

            l += log(2.0) - log(df) - log(PI) - log(om1) - log(om2) - 0.5 * log(oneminrhotwo);
            l += randnumbers::lngamma_exact(dfplustwohalf) - randnumbers::lngamma_exact(dfhalf) - dfplustwohalf * log(1.0 + Q / df);
            l += log(gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0));
            // Compute log-likelihood (complete) HERE sind 0 und 1 richtig oder anders

            *deviance = -2*l;
        }
    }


    double DISTR_bivst_xi::get_intercept_start(void)
    {
        return 0;
    }

    void DISTR_bivst_xi::compute_param_mult(vector<double *>  linpred,double * param)
    {
        *param = (*linpred[6+pos]); // <--- c.f. linpred info
    }

    // ONLY FOR THE MAIN PARAMETER:
    double DISTR_bivst_xi::cdf_mult(vector<double *> response,
                                    vector<double *> param,
                                    vector<double *> weight,
                                    vector<datamatrix *> aux)


    {
        return 0; // calculate cdf for observation using param!
    }

    // ONLY FOR THE MAIN PARAMETER:
    double DISTR_bivst_xi::pdf_mult(vector<double *> response,
                                    vector<double *> param,
                                    vector<double *> weight,
                                    vector<datamatrix *> aux)
    {
        return 0; // usually set to be zero
    }

    void DISTR_bivst_xi::set_worklin(void)
    {

        DISTR_gamlss::set_worklin();

        response2p = response2.getV();

    }



    void DISTR_bivst_xi::modify_worklin(void)
    {

        DISTR_gamlss::modify_worklin();

        if (counter<nrobs-1)
        {
            response2p++;
        }

    }


    double DISTR_bivst_xi::loglikelihood_weightsone(double * response,
                                                    double * linpred)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double df = exp(*worklin[0]);
        double al2 = (*worklin[1]);
        double al1 = (*worklin[2]);
        double r = (*worklin[3]) / sqrt(1 + pow((*worklin[3]), 2));
        double r2 = r * r;
        double om2 = exp((*worklin[4]));
        double om1 = exp((*worklin[5]));
        double xi2 = (*worklin[6]);
        double xi1 = (*linpred);

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;

        double l = 0;

        l += - dfplustwohalf * log(1.0 + Q / df);
        l += log(gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0));

        modify_worklin();

        return l;

    }


    void DISTR_bivst_xi::compute_iwls_wweightschange_weightsone(
                                                                double * response,
                                                                double * linpred,
                                                                double * workingweight,
                                                                double * workingresponse,
                                                                double & like,
                                                                const bool & compute_like)
    {
        if (counter==0)
        {
            set_worklin();
        }

        double df = exp(*worklin[0]);
        double al2 = (*worklin[1]);
        double al1 = (*worklin[2]);
        double r = (*worklin[3]) / sqrt(1 + pow((*worklin[3]), 2));
        double r2 = r * r;
        double om2 = exp((*worklin[4]));
        double om1 = exp((*worklin[5]));
        double xi2 = (*worklin[6]);
        double xi1 = (*linpred);

        double oneminrhotwo = 1.0 - r2;
        double z1 = (*response - xi1) / om1;
        double z2 = (*response2p - xi2) / om2;
        double Q = (z1 * z1 - 2 * r * z1 * z2 + z2 * z2) / oneminrhotwo;
        double dfplustwohalf = (df + 2.0) * 0.5;

        double tval = gsl_ran_tdist_pdf((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);
        double Tval = gsl_cdf_tdist_P((al1 * z1 + al2 * z2) * sqrt((df + 2.0) / (df + Q)), df + 2.0);

        double nu = - (- (df + 2.0) / 2.0 * (2.0 * z1 - 2 * r * z2) / (1.0 + Q / df) / df / oneminrhotwo
                       + tval / Tval * (al1 * sqrt((df + 2.0) / (df + Q)) - (al1 * z1 + al2 * z2) * sqrt(df + 2.0) * (z1 - r * z2) / oneminrhotwo / sqrt(df + Q) / (df + Q)));

        if(nu != nu){
            nu = 0.0;
        } // prevent nans (might only happen if *workingweight is fixed to a small value

        *workingweight = ef[max(-9, min(9, (int)nearbyint(r * 10))) + 9][max(-5, min(5, (int)nearbyint(al1*0.5))) + 5][max(-5, min(5, (int)nearbyint(al2*0.5))) + 5][max(-10, min(24, (int)nearbyint(log(df) * 5))) + 10]; // w

        *workingresponse = *linpred + nu/(*workingweight);

        if (compute_like)
        {

            like += - dfplustwohalf * log(1.0 + Q / df) + log(Tval); // loglikelihood

        }


        modify_worklin();

    }


    // ONLY FOR THE MAIN PARAMETER:
    void DISTR_bivst_xi::compute_mu_mult(vector<double *> linpred,vector<double *> response,double * mu)
    {
        *mu = ((*linpred[predstart_mumult+5+pos])); // <--- depends on the number of parameters
    }


    void DISTR_bivst_xi::outoptions(void)
    {
        DISTR::outoptions();
        optionsp->out("  Response function (xi): identity\n");
        optionsp->out("\n");
        optionsp->out("\n");
    }


    void DISTR_bivst_xi::update_end(void)
    {


        // helpmat1 stores (eta_mu)

        double * worklin;
        if (linpred_current==1)
            worklin = linearpred1.getV();
        else
            worklin = linearpred2.getV();

        double * pmu = helpmat1.getV();

        unsigned i;
        for (i=0;i<nrobs;i++,pmu++,worklin++)
        {
            *pmu = (*worklin);
            //    double t = 0;:
        }
    }
