/* BayesX - Software for Bayesian Inference in
Structured Additive Regression Models.
Copyright (C) 2011  Christiane Belitz, Andreas Brezger,
Thomas Kneib, Stefan Lang, Nikolaus Umlauf

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */


#include"../export_type.h"
#include"statmat.h"
#include"Random.h"
#include"GENERAL_OPTIONS.h"
#include"FC.h"
#include"distr_gamlss.h"

#include"data_st_ef_xi.h"
#include"data_st_ef_omega.h"
#include"data_st_ef_alpha.h"
#include"data_st_ef_df.h"
#include"data_st_iota.h"
#include"data_bivst_ef_xi.h"
#include"data_bivst_ef_omega.h"
#include"data_bivst_ef_rho.h"
#include"data_bivst_ef_alpha.h"
#include"data_bivst_ef_df.h"
#include"data_bivst_iota.h"
#include"func_sn.h"

namespace MCMC
{

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_sndp_alpha ---------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_sndp_alpha : public DISTR_gamlss
    {

    protected:


    public:


        // DEFAULT CONSTRUCTOR

        DISTR_sndp_alpha(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_sndp_alpha(GENERAL_OPTIONS * o, const datamatrix & r,
                         const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_sndp_alpha(const DISTR_sndp_alpha & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_sndp_alpha & operator=(const DISTR_sndp_alpha & nd);

        // DESTRUCTOR

        ~DISTR_sndp_alpha() {}


        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_sndp_omega ---------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_sndp_omega : public DISTR_gamlss
    {

    protected:


    public:

        // DEFAULT CONSTRUCTOR

        DISTR_sndp_omega(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_sndp_omega(GENERAL_OPTIONS * o, const datamatrix & r,
                         const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_sndp_omega(const DISTR_sndp_omega & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_sndp_omega & operator=(const DISTR_sndp_omega & nd);

        // DESTRUCTOR

        ~DISTR_sndp_omega() {}


        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };


    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_sndp_xi ------------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_sndp_xi : public DISTR_gamlss
    {

    protected:


    public:

        // ONLY FOR THE MAIN PARAMETER:

        void check_errors(void);



        // DEFAULT CONSTRUCTOR

        DISTR_sndp_xi(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_sndp_xi(GENERAL_OPTIONS * o, const datamatrix & r,
                      const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_sndp_xi(const DISTR_sndp_xi & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_sndp_xi & operator=(const DISTR_sndp_xi & nd);

        // DESTRUCTOR

        ~DISTR_sndp_xi() {}


        // ONLY FOR THE MAIN PARAMETER:

        void compute_deviance_mult(vector<double *> response,
                                   vector<double *> weight,
                                   vector<double *> linpred,
                                   double * deviance,
                                   vector<datamatrix*> aux);



        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        // ONLY FOR THE MAIN PARAMETER:

        double cdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        // ONLY FOR THE MAIN PARAMETER:

        double pdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        // ONLY FOR THE MAIN PARAMETER:

        void compute_mu_mult(vector<double *> linpred,vector<double *> response,double * mu);


        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_sncp_gamma ---------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_sncp_gamma : public DISTR_gamlss
    {

    protected:


    public:

        // DEFAULT CONSTRUCTOR

        DISTR_sncp_gamma(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_sncp_gamma(GENERAL_OPTIONS * o, const datamatrix & r,
                         const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_sncp_gamma(const DISTR_sncp_gamma & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_sncp_gamma & operator=(const DISTR_sncp_gamma & nd);

        // DESTRUCTOR

        ~DISTR_sncp_gamma() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);

        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_sncp_sigma ---------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_sncp_sigma : public DISTR_gamlss
    {

    protected:


    public:

        // DEFAULT CONSTRUCTOR

        DISTR_sncp_sigma(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_sncp_sigma(GENERAL_OPTIONS * o, const datamatrix & r,
                         const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_sncp_sigma(const DISTR_sncp_sigma & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_sncp_sigma & operator=(const DISTR_sncp_sigma & nd);

        // DESTRUCTOR

        ~DISTR_sncp_sigma() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);

        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_sncp_mu ------------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_sncp_mu : public DISTR_gamlss
    {

    protected:


    public:

        // ONLY FOR THE MAIN PARAMETER:

        void check_errors(void);



        // DEFAULT CONSTRUCTOR

        DISTR_sncp_mu(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_sncp_mu(GENERAL_OPTIONS * o, const datamatrix & r,
                      const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_sncp_mu(const DISTR_sncp_mu & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_sncp_mu & operator=(const DISTR_sncp_mu & nd);

        // DESTRUCTOR

        ~DISTR_sncp_mu() {}


        // ONLY FOR THE MAIN PARAMETER:

        void compute_deviance_mult(vector<double *> response,
                                   vector<double *> weight,
                                   vector<double *> linpred,
                                   double * deviance,
                                   vector<datamatrix*> aux);



        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        // ONLY FOR THE MAIN PARAMETER:

        double cdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        // ONLY FOR THE MAIN PARAMETER:

        double pdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        // ONLY FOR THE MAIN PARAMETER:

        void compute_mu_mult(vector<double *> linpred,vector<double *> response,double * mu);


        void outoptions(void);

        void update_end(void);

    };


    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivsn_alpha --------------------------------------
    //------------------------------------------------------------------------------


    class __EXPORT_TYPE DISTR_bivsn_alpha : public DISTR_gamlss
    {

    protected:
        unsigned pos;

        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;

        // DEFAULT CONSTRUCTOR

        DISTR_bivsn_alpha(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivsn_alpha(GENERAL_OPTIONS * o, const datamatrix & r, unsigned & p,
                          const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivsn_alpha(const DISTR_bivsn_alpha & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivsn_alpha & operator=(const DISTR_bivsn_alpha & nd);

        // DESTRUCTOR

        ~DISTR_bivsn_alpha() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivsn_rho ----------------------------------------
    //------------------------------------------------------------------------------


    class __EXPORT_TYPE DISTR_bivsn_rho : public DISTR_gamlss
    {

    protected:
        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;

        // DEFAULT CONSTRUCTOR

        DISTR_bivsn_rho(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivsn_rho(GENERAL_OPTIONS * o, const datamatrix & r,
                        const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivsn_rho(const DISTR_bivsn_rho & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivsn_rho & operator=(const DISTR_bivsn_rho & nd);

        // DESTRUCTOR

        ~DISTR_bivsn_rho() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivsn_omega --------------------------------------
    //------------------------------------------------------------------------------


    class __EXPORT_TYPE DISTR_bivsn_omega : public DISTR_gamlss
    {

    protected:
        unsigned pos;

        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;

        // DEFAULT CONSTRUCTOR

        DISTR_bivsn_omega(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivsn_omega(GENERAL_OPTIONS * o, const datamatrix & r, unsigned & p,
                          const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivsn_omega(const DISTR_bivsn_omega & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivsn_omega & operator=(const DISTR_bivsn_omega & nd);

        // DESTRUCTOR

        ~DISTR_bivsn_omega() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivsn_xi -----------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_bivsn_xi : public DISTR_gamlss
    {

    protected:
        unsigned pos;

        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;

        // ONLY FOR THE MAIN PARAMETER:
        void check_errors(void);

        // DEFAULT CONSTRUCTOR
        DISTR_bivsn_xi(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivsn_xi(GENERAL_OPTIONS * o, const datamatrix & r, unsigned & p,
                       const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivsn_xi(const DISTR_bivsn_xi & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivsn_xi & operator=(const DISTR_bivsn_xi & nd);

        // DESTRUCTOR

        ~DISTR_bivsn_xi() {}


        // ONLY FOR THE MAIN PARAMETER:

        void compute_deviance_mult(vector<double *> response,
                                   vector<double *> weight,
                                   vector<double *> linpred,
                                   double * deviance,
                                   vector<datamatrix*> aux);



        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        // ONLY FOR THE MAIN PARAMETER:

        double cdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        // ONLY FOR THE MAIN PARAMETER:

        double pdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        // ONLY FOR THE MAIN PARAMETER:

        void compute_mu_mult(vector<double *> linpred,vector<double *> response,double * mu);


        void outoptions(void);

        void update_end(void);

    };


    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_st_df --------------------------------------------
    //------------------------------------------------------------------------------


    class __EXPORT_TYPE DISTR_st_df : public DISTR_gamlss
    {

    protected:


    public:

        // array to store the expected fisher information, 701 because it covers -2 to 5 with 0.01 steps (for eta_df)
        double ef[701];

        double io[201][71];

        // DEFAULT CONSTRUCTOR

        DISTR_st_df(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_st_df(GENERAL_OPTIONS * o, const datamatrix & r,
                    const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_st_df(const DISTR_st_df & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_st_df & operator=(const DISTR_st_df & nd);

        // DESTRUCTOR

        ~DISTR_st_df() {}


        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);



        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_st_alpha -----------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_st_alpha : public DISTR_gamlss
    {

    protected:


    public:
        double ef[201][71];

        // DEFAULT CONSTRUCTOR

        DISTR_st_alpha(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_st_alpha(GENERAL_OPTIONS * o, const datamatrix & r,
                       const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_st_alpha(const DISTR_st_alpha & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_st_alpha & operator=(const DISTR_st_alpha & nd);

        // DESTRUCTOR

        ~DISTR_st_alpha() {}


        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);



        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_st_omega -----------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_st_omega : public DISTR_gamlss
    {

    protected:


    public:
        double ef[201][71];

        // DEFAULT CONSTRUCTOR

        DISTR_st_omega(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_st_omega(GENERAL_OPTIONS * o, const datamatrix & r,
                       const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_st_omega(const DISTR_st_omega & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_st_omega & operator=(const DISTR_st_omega & nd);

        // DESTRUCTOR

        ~DISTR_st_omega() {}



        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);



        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_st_xi --------------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_st_xi : public DISTR_gamlss
    {

    protected:


    public:
        double ef[201][71];
        // ONLY FOR THE MAIN PARAMETER:

        void check_errors(void);



        // DEFAULT CONSTRUCTOR

        DISTR_st_xi(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_st_xi(GENERAL_OPTIONS * o, const datamatrix & r,
                    const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_st_xi(const DISTR_st_xi & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_st_xi & operator=(const DISTR_st_xi & nd);

        // DESTRUCTOR

        ~DISTR_st_xi() {}


        // ONLY FOR THE MAIN PARAMETER:

        void compute_deviance_mult(vector<double *> response,
                                   vector<double *> weight,
                                   vector<double *> linpred,
                                   double * deviance,
                                   vector<datamatrix*> aux);



        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        // ONLY FOR THE MAIN PARAMETER:

        double cdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        // ONLY FOR THE MAIN PARAMETER:

        double pdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        // ONLY FOR THE MAIN PARAMETER:

        void compute_mu_mult(vector<double *> linpred,vector<double *> response,double * mu);


        void outoptions(void);

        void update_end(void);

    };


    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_df --------------------------------------
    //------------------------------------------------------------------------------


    class __EXPORT_TYPE DISTR_bivst_df : public DISTR_gamlss
    {

    protected:
        unsigned pos;

        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;
        double ef[701];
        double iota[1001][71];

        // DEFAULT CONSTRUCTOR

        DISTR_bivst_df(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivst_df(GENERAL_OPTIONS * o, const datamatrix & r,
                          const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivst_df(const DISTR_bivst_df & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivst_df & operator=(const DISTR_bivst_df & nd);

        // DESTRUCTOR

        ~DISTR_bivst_df() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };


    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_alpha --------------------------------------
    //------------------------------------------------------------------------------


    class __EXPORT_TYPE DISTR_bivst_alpha : public DISTR_gamlss
    {

    protected:
        unsigned pos;

        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;
        double ef[19][11][11][35];

        // DEFAULT CONSTRUCTOR

        DISTR_bivst_alpha(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivst_alpha(GENERAL_OPTIONS * o, const datamatrix & r, unsigned & p,
                          const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivst_alpha(const DISTR_bivst_alpha & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivst_alpha & operator=(const DISTR_bivst_alpha & nd);

        // DESTRUCTOR

        ~DISTR_bivst_alpha() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_rho ----------------------------------------
    //------------------------------------------------------------------------------


    class __EXPORT_TYPE DISTR_bivst_rho : public DISTR_gamlss
    {

    protected:
        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;
        double ef[19][11][11][35];

        // DEFAULT CONSTRUCTOR

        DISTR_bivst_rho(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivst_rho(GENERAL_OPTIONS * o, const datamatrix & r,
                        const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivst_rho(const DISTR_bivst_rho & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivst_rho & operator=(const DISTR_bivst_rho & nd);

        // DESTRUCTOR

        ~DISTR_bivst_rho() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_omega --------------------------------------
    //------------------------------------------------------------------------------


    class __EXPORT_TYPE DISTR_bivst_omega : public DISTR_gamlss
    {

    protected:
        unsigned pos;

        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;
        double ef[19][11][11][35];

        // DEFAULT CONSTRUCTOR

        DISTR_bivst_omega(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivst_omega(GENERAL_OPTIONS * o, const datamatrix & r, unsigned & p,
                          const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivst_omega(const DISTR_bivst_omega & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivst_omega & operator=(const DISTR_bivst_omega & nd);

        // DESTRUCTOR

        ~DISTR_bivst_omega() {}

        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        void outoptions(void);

        void update_end(void);

    };

    //------------------------------------------------------------------------------
    //-------------- CLASS: DISTR_bivst_xi -----------------------------------------
    //------------------------------------------------------------------------------

    class __EXPORT_TYPE DISTR_bivst_xi : public DISTR_gamlss
    {

    protected:
        unsigned pos;

        void set_worklin(void);
        void modify_worklin(void);

    public:
        datamatrix response2;
        double * response2p;
        double ef[19][11][11][35];

        // ONLY FOR THE MAIN PARAMETER:
        void check_errors(void);

        // DEFAULT CONSTRUCTOR
        DISTR_bivst_xi(void) : DISTR_gamlss()
        {
        }

        // CONSTRUCTOR

        DISTR_bivst_xi(GENERAL_OPTIONS * o, const datamatrix & r, unsigned & p,
                       const datamatrix & w=datamatrix());

        // COPY CONSTRUCTOR

        DISTR_bivst_xi(const DISTR_bivst_xi & nd);

        // OVERLOADED ASSIGNMENT OPERATOR

        const DISTR_bivst_xi & operator=(const DISTR_bivst_xi & nd);

        // DESTRUCTOR

        ~DISTR_bivst_xi() {}


        // ONLY FOR THE MAIN PARAMETER:

        void compute_deviance_mult(vector<double *> response,
                                   vector<double *> weight,
                                   vector<double *> linpred,
                                   double * deviance,
                                   vector<datamatrix*> aux);



        double get_intercept_start(void);

        void compute_param_mult(vector<double *>  linpred,double * param);

        // ONLY FOR THE MAIN PARAMETER:

        double cdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        // ONLY FOR THE MAIN PARAMETER:

        double pdf_mult(vector<double *> response,
                        vector<double *> param,
                        vector<double *> weight,
                        vector<datamatrix *> aux);


        double loglikelihood_weightsone(double * response, double * linpred);

        void compute_iwls_wweightschange_weightsone(double * response,
                                                    double * linpred,
                                                    double * workingweight,
                                                    double * workingresponse,
                                                    double & like,
                                                    const bool & compute_like);


        // ONLY FOR THE MAIN PARAMETER:

        void compute_mu_mult(vector<double *> linpred,vector<double *> response,double * mu);


        void outoptions(void);

        void update_end(void);

    };

}
